// GfxEditView.cpp : implementation file
//

#include "stdafx.h"
#include "searchr.h"
#include "GfxEditView.h"
#include "TableDoc.h"
#include "SearchRDoc.h"

#include "TileEditCtrl.h"

const ID_TIMER = 1;

#define CHAR_PADDING 2

#define WIDTH_BYTES(bits) ((((bits) + 31) / 32) * 4)

BOOL ready_stuff;
BYTE* m_pBits;
CBitmap m_Bitmap;
CDC* m_pMemDC;
CWindowDC* m_pWinDC;

/////////////////////////////////////////////////////////////////////////////
// CGfxEditView

IMPLEMENT_DYNCREATE(CGfxEditView, CView)

CGfxEditView::CGfxEditView()
{
	m_dwStartAddr = 0;
	m_nEditDigit = 0;
	m_selStart = m_selEnd = -1;
	m_editPos = m_viewPos = m_nHorzScroll = 0;

	m_bMouseDown = m_bMouseMove = FALSE;
	m_bTimer = FALSE;

	m_showAddress = TRUE;
	m_allowChangeSize = FALSE;
	m_digitsInAddress = 8;
	m_digitsInData = 2;
	m_font_height = 16;

	ready_stuff = FALSE;

	m_planes = 4;
	m_width = 8;
	m_height = 8;
	m_columns = 32;
	m_byte_pos = 0;

	int i;
	for(i = 0; i < COLOUR_COUNT; i ++)
		m_colour_data[i] = gfx_edit_def_colours[i];

	RecalcLayout();
}

CGfxEditView::~CGfxEditView()
{
}


BEGIN_MESSAGE_MAP(CGfxEditView, CView)
	//{{AFX_MSG_MAP(CGfxEditView)
	ON_WM_VSCROLL()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SELECT_ALL, OnUpdateEditSelectAll)
	ON_WM_CHAR()
	ON_WM_LBUTTONDOWN()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_GETDLGCODE()
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGfxEditView drawing

void CGfxEditView::OnDraw(CDC* pdc)
{
	CRect rc;
	GetClientRect(rc);

	CDC	dc;
	dc.CreateCompatibleDC(pdc);

	CBitmap bm;
	bm.CreateCompatibleBitmap(pdc, rc.Width(), rc.Height());
	dc.SelectObject(bm);
	
	if(!p_doc->data)
	{
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc, &bkBrush);
		pdc->BitBlt(0, 0, rc.Width(), rc.Height(), &dc, 0, 0, SRCCOPY);
		return;
	}

	CFont font;
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, 0 );
	font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("MS Gothic") );
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Januschan JOME Regular") );
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Arial") );

	CFont* oldFont = dc.SelectObject ( &font );

	dc.SetBoundsRect(&rc, DCB_DISABLE);

	CString strText;
	int nAddr = m_viewPos + m_byte_pos;
	BYTE* buf;

	{
		CRect rc2 ( -(m_nHorzScroll*m_cellSize.cx), rc.top, (m_cellSize.cx/2)-(m_nHorzScroll*m_cellSize.cx), rc.bottom );
		CBrush bkBrush(m_colour_data[BACK_COLOUR]);
		dc.FillRect(rc2, &bkBrush);
	}
	{
		CRect rc2 ( (m_cellSize.cx/2) - (m_nHorzScroll * m_cellSize.cx), m_charCountWindow.cy*m_cellSize.cy, rc.right, rc.bottom);
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc2, &bkBrush);
	}
	{
		CRect rc2 ( (m_cellSize.cx/2) + ((m_charCountView.cx - m_nHorzScroll) * m_cellSize.cx), rc.top, rc.right, m_charCountWindow.cy*m_cellSize.cy);
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc2, &bkBrush);
	}

	buf = p_doc->data;

	for ( int i = 0; i < m_charCountWindow.cy; i++ )
	{
		dc.SetTextColor((m_colour_data[HEX_COLOUR]));
		dc.SetBkColor((m_colour_data[BACK_COLOUR]));

		strText = ' ';
		//dc.TextOut( (m_asciiMargin-m_nHorzScroll-1)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
		dc.TextOut( (m_charCountView.cx-m_nHorzScroll-1)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );

		if ( m_showAddress )
		{
			// draw address field
			if (DWORD(nAddr) < p_doc->size)
				strText.Format ( _T("%08X  "), nAddr + m_dwStartAddr );
			else
				strText.Format ( _T("%8c  "), ' ' );

			dc.SetTextColor(m_colour_data[ADDRESS_COLOUR]);
			dc.SetBkColor(m_colour_data[BACK_COLOUR]);
			dc.TextOut( -m_nHorzScroll*m_cellSize.cx+m_cellSize.cx/2, i*m_height*2, strText );
		}
		{
			// draw hex data
			DWORD dwPos = m_viewPos + m_byte_pos + (i * ((m_width / 8) * m_planes * m_height * m_columns));
			if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
			{
				dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
				dc.SetBkColor(GetSysColor(COLOR_HIGHLIGHT));
			}
			else
			{
				dc.SetTextColor(m_colour_data[HEX_COLOUR]);
				dc.SetBkColor(m_colour_data[BACK_COLOUR]);
			}

			DWORD xx, yy, zz;
			DWORD plane_data[8], aa, pp;

			for(zz = 0; zz < m_columns; zz ++)
			{
				for(yy = 0; yy < m_height; yy ++)
				{
					for(pp = 0; pp < m_planes; pp ++)
					{
						plane_data[pp]  = buf[(dwPos + (yy * (m_width / 8) * m_planes) + (zz * (m_width / 8) * m_planes * m_height) + ((m_width / 8) * pp) + 0) ^ 1] << 24;
						plane_data[pp] |= buf[(dwPos + (yy * (m_width / 8) * m_planes) + (zz * (m_width / 8) * m_planes * m_height) + ((m_width / 8) * pp) + 1) ^ 1] << 16;
						plane_data[pp] |= buf[(dwPos + (yy * (m_width / 8) * m_planes) + (zz * (m_width / 8) * m_planes * m_height) + ((m_width / 8) * pp) + 2) ^ 1] << 8;
						plane_data[pp] |= buf[(dwPos + (yy * (m_width / 8) * m_planes) + (zz * (m_width / 8) * m_planes * m_height) + ((m_width / 8) * pp) + 3) ^ 1] << 0;
					}

					BYTE* pRow = (m_pBits + (WIDTH_BYTES((DWORD)m_columns * m_width * 8) * (((m_charCountWindow.cy * m_height) - 1) - (yy + (i*(m_height)))))) + (zz * m_width);

					for(xx = 0; xx < m_width; xx ++)
					{
						aa = 0;

						for(pp = 0; pp < m_planes; pp ++)
						{
							aa |= ((plane_data[pp] & 0x80000000) >> 31) << pp;
							plane_data[pp] <<= 1;
						}

						aa <<= 8 - m_planes;

						*pRow++ = aa;

					}
				}

			}

		}

		nAddr += (m_width / 8) * m_planes * m_height * m_columns;
	}

	pdc->BitBlt(0, 0, ((m_addrMargin - m_nHorzScroll) * m_cellSize.cx)+(m_cellSize.cx/2), rc.Height(), &dc, 0, 0, SRCCOPY);

	CBitmap* pOldBitmap = dc.SelectObject( &m_Bitmap );
	pdc->StretchBlt(((m_addrMargin - m_nHorzScroll) * m_cellSize.cx)+(m_cellSize.cx/2), 0, m_columns * m_width * 2, m_charCountWindow.cy * m_height * 2, &dc, 0, 0, m_columns * m_width, m_charCountWindow.cy * m_height, SRCCOPY);
	dc.SelectObject(pOldBitmap);
}


void CGfxEditView::RecalcLayout()
{
	if ( m_hWnd == 0 ) return;		// invisible window

	CFont font;
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, 0 );
	font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("MS Gothic") );
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Januschan JOME Regular") );
	//font.CreateFont ( m_font_height, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Arial") );

	CDC* pdc = GetDC();
	ASSERT ( pdc );

	CFont* oldFont = pdc->SelectObject(&font);

	// determine char size
	m_cellSize = pdc->GetTextExtent(_T("0"));
	pdc->LPtoDP ( &m_cellSize );
	pdc->SelectObject ( oldFont );

	GetClientRect ( m_rcClient );

	if(m_rcClient.Height() == 0)
	{
		ReleaseDC ( pdc );
		return;
	}

	// compute entire rect of the control, including scroll bars
	if (GetStyle() & WS_VSCROLL)
		m_rcClient.right += ::GetSystemMetrics(SM_CXVSCROLL);
	if (GetStyle() & WS_HSCROLL)
		m_rcClient.bottom += ::GetSystemMetrics(SM_CYHSCROLL);

	// before calculations
	BOOL bShowSbVert = false;	// need to show vertical scroll bar?
	BOOL bShowSbHorz = false;	// need to show horizontal scroll bar?

	// after calculations
	BOOL bShowSbVert2 = false;	
	BOOL bShowSbHorz2 = false;	

	do
	{
		bShowSbVert = bShowSbVert2;
		bShowSbHorz = bShowSbHorz2;

		// first pass - compute params when scrollbars are hidden
		// next passes (0, 1, or 2) - recalulate params if client rect was shrunk after showing scrollbars
		m_charCountWindow.cx = (m_rcClient.Width() - m_cellSize.cx/2) / m_cellSize.cx;
		m_charCountWindow.cy = m_rcClient.Height() / (m_height * 2);

		m_charCountView.cx = 0;

		if (m_showAddress)
			m_charCountView.cx += m_digitsInAddress + 2;

		m_addrMargin = m_charCountView.cx;

		m_charCountView.cx += ((m_width / 8) * m_columns) * 2;

		m_charCountView.cy = p_doc->size / (((m_width / 8) * m_planes * m_height * m_columns)*(m_digitsInData/2)) + 1;

		if ( m_charCountView.cx > m_charCountWindow.cx && !bShowSbHorz )
		{
			m_rcClient.bottom -= ::GetSystemMetrics(SM_CYHSCROLL);
			bShowSbHorz2 = true;
		}
		if ( m_charCountView.cy > m_charCountWindow.cy && !bShowSbVert )
		{
			m_rcClient.right -= ::GetSystemMetrics(SM_CXVSCROLL);
			bShowSbVert2 = true;
		}
	} while ((bShowSbHorz != bShowSbHorz2) || (bShowSbVert != bShowSbVert2));

//	m_nHorzScroll = 0;
	m_viewPos /= ((m_width / 8) * m_planes * m_height);
	m_viewPos *= ((m_width / 8) * m_planes * m_height);
	NormalizeEditPos();
	UpdateScrollBars();

	if (!(GetStyle() & WS_VSCROLL))
		m_viewPos = 0;

	if(!p_doc->data)
	{
		ReleaseDC ( pdc );
		return;
	}

	//if(ready_stuff == FALSE)
	{
		if(ready_stuff == TRUE)
		{
			m_Bitmap.Detach();
			m_pMemDC->DeleteDC();
			//delete [] (BYTE*) m_pMemDC;
		}

		ready_stuff = TRUE;

		LPBITMAPINFO lpbi;

		// Fill in the BITMAPINFOHEADER
		lpbi = (LPBITMAPINFO) new BYTE[sizeof(BITMAPINFOHEADER) + (256 * sizeof(RGBQUAD))];
		lpbi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		lpbi->bmiHeader.biWidth = (LONG) (m_columns * m_width);
		lpbi->bmiHeader.biHeight = m_charCountWindow.cy * ((LONG) m_height);
		lpbi->bmiHeader.biPlanes = 1;
		lpbi->bmiHeader.biBitCount = 8;
		lpbi->bmiHeader.biCompression = BI_RGB;
		lpbi->bmiHeader.biSizeImage = WIDTH_BYTES((DWORD)m_columns * m_width * 8) * ((DWORD) m_charCountWindow.cy) * m_height;
		lpbi->bmiHeader.biXPelsPerMeter = 0;
		lpbi->bmiHeader.biYPelsPerMeter = 0;
		lpbi->bmiHeader.biClrUsed = 0;
		lpbi->bmiHeader.biClrImportant = 0;

		// Fill in the color table
		UINT uUsage = DIB_RGB_COLORS;
		int i;
		for(i = 0; i < 256; i++)
		{
			lpbi->bmiColors[i].rgbRed = i;
			lpbi->bmiColors[i].rgbGreen = i;
			lpbi->bmiColors[i].rgbBlue = i;
		}

		HBITMAP hBitmap = CreateDIBSection(pdc->m_hDC, lpbi, uUsage, (void **)&m_pBits, NULL, 0 );

		delete [] (BYTE *)lpbi;

		ASSERT(hBitmap != NULL);

		m_Bitmap.Attach( hBitmap );

		for (int y = 0; y < m_charCountWindow.cy * m_height; y++ )
		{
			BYTE* pRow = (m_pBits + WIDTH_BYTES((DWORD)m_columns * m_width * 8) * y);
			for (int x = 0; x < (m_columns * m_width); x++ )
				*pRow++ = 0;
		}

		m_pMemDC = new CDC;
		VERIFY( m_pMemDC->CreateCompatibleDC(pdc) );

//		m_pOldPalette = pdc->SelectPalette( &m_Palette, FALSE );
//		m_pWinDC->RealizePalette();
//		m_pOldBitmap = m_pMemDC->SelectObject( &m_Bitmap );
	}

	ReleaseDC ( pdc );

	CreateSolidCaret(m_width * 2, m_height * 2);
	PlaceCaret(true);
	if(!m_bMouseDown)
		ShowCaret();
}

BOOL CGfxEditView::NormalizeEditPos()
{
	BOOL bRepaint = false;
	DWORD dwDataSize = p_doc->size/(m_digitsInData/2);

	if ( m_editPos < 0 )
		m_editPos = 0;
	else if ( DWORD(m_editPos) >= dwDataSize )
	{
		m_editPos = dwDataSize - 1;
		m_nEditDigit = 0;
	}

	// change m_viewPos if m_editPos is moved out of the visible area
	if((m_viewPos) > m_editPos)
	{
		m_viewPos = (m_viewPos % ((m_width / 8) * m_planes * m_height * m_columns)) + (m_editPos / ((m_width / 8) * m_planes * m_height * m_columns) * ((m_width / 8) * m_planes * m_height * m_columns));
		if((m_viewPos) > m_editPos)
		{
			m_viewPos -= (m_width / 8) * m_planes * m_height * m_columns;
		}
		bRepaint = true;
	}
	else if ( m_viewPos + ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy) <= m_editPos )
	{
		m_viewPos = (m_viewPos % ((m_width / 8) * m_planes * m_height * m_columns)) + ((m_editPos - ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy)) / ((m_width / 8) * m_planes * m_height * m_columns) * ((m_width / 8) * m_planes * m_height * m_columns));
		if( m_viewPos + ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy) <= m_editPos )
		{
			m_viewPos += (m_width / 8) * m_planes * m_height * m_columns;
		}

		//m_viewPos = (m_editPos / ((m_width / 8) * m_planes * m_height) - (m_charCountWindow.cy - 1)) * ((m_width / 8) * m_planes * m_height);
		bRepaint = true;
	}

	// if m_viewPos is too near to the end of data, decrease m_viewPos so entire page is displayed
	// (this may happen after deleting large selection or changing properties)
	if ( DWORD(m_viewPos) > p_doc->size / (m_digitsInData/2) - ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy))
	{
		m_viewPos = p_doc->size / (m_digitsInData/2) / ((m_width / 8) * m_planes * m_height * m_columns) * ((m_width / 8) * m_planes * m_height * m_columns) - ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy);
		if ( m_viewPos < 0 ) m_viewPos = 0;
	}

	int nPosX;		// x coordinate of caret in view
	nPosX = m_addrMargin + m_editPos % ((m_width / 8) * m_planes * m_height * m_columns) * (m_digitsInData+1);
	
	if ( nPosX < m_nHorzScroll )
	{
		m_nHorzScroll = nPosX;
		bRepaint = true;
	}
	else
	{
		nPosX += m_digitsInData;	// to make entire number visible
		if ( nPosX-m_nHorzScroll >= m_charCountWindow.cx )
		{
			m_nHorzScroll = nPosX - m_charCountWindow.cx;
			bRepaint = true;
		}
	}
	return bRepaint;
}

BOOL CGfxEditView::UpdateScrollBars()
{
	BOOL bRepaint = false;
	SCROLLINFO si, prevSI;

	si.cbSize = prevSI.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	si.nMin = 0;
	si.nMax = m_charCountView.cy - 1;
	si.nPage = m_charCountWindow.cy;
	si.nPos = (m_viewPos + ((m_width / 8) * m_planes * m_height * m_columns) - 1) / ((m_width / 8) * m_planes * m_height * m_columns);

	if ( !bRepaint && ( !GetScrollInfo(SB_VERT, &prevSI, SIF_POS ) || si.nPos != prevSI.nPos ))
		// if scrollbar is about to show or its position changed, the control needs repainting
		bRepaint = true;

	if(si.nMax >= int(si.nPage))
	{
		ShowScrollBar(SB_VERT);
	}
	else
	{
		ShowScrollBar(SB_VERT, FALSE);
	}
	VERIFY(SetScrollInfo(SB_VERT, &si, TRUE));

	si.fMask = SIF_ALL;
	si.nMin = 0;
	si.nMax = m_charCountView.cx - 1;
	si.nPage = m_charCountWindow.cx;
	si.nPos = m_nHorzScroll = 0;

	if ( (si.nMax > (int)si.nPage) && ((si.nPos+int(si.nPage)) > si.nMax) )
		si.nPos = m_nHorzScroll = 0; //(si.nMax - si.nPage) - 1;
	if ( !bRepaint && ( !GetScrollInfo(SB_HORZ, &prevSI, SIF_POS ) || si.nPos != prevSI.nPos ))
		// if scrollbar is about to show or its position changed, the control needs repainting
		bRepaint = true;

	if(si.nMax >= (int)si.nPage )
	{
		ShowScrollBar(SB_HORZ);
	}
	else
	{
		ShowScrollBar(SB_HORZ, FALSE);
	}

	VERIFY(SetScrollInfo(SB_HORZ, &si, TRUE));
	return bRepaint;
}

/////////////////////////////////////////////////////////////////////////////
// CGfxEditView diagnostics

#ifdef _DEBUG
void CGfxEditView::AssertValid() const
{
	CView::AssertValid();
}

void CGfxEditView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGfxEditView message handlers

void CGfxEditView::OnVScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/) 
{
	int nPrevViewPos = m_viewPos;

	switch(nSBCode)
	{
	case SB_LINEDOWN:
		m_viewPos += ((m_width / 8) * m_planes * m_height * m_columns);
		break;
	case SB_LINEUP:
		m_viewPos -= ((m_width / 8) * m_planes * m_height * m_columns);
		break;
	case SB_PAGEDOWN:
		m_viewPos += ((m_width / 8) * m_planes * m_height * m_columns)*m_charCountWindow.cy;
		break;
	case SB_PAGEUP:
		m_viewPos -= ((m_width / 8) * m_planes * m_height * m_columns)*m_charCountWindow.cy;
		break;
	case SB_THUMBTRACK:
		SCROLLINFO si;

		si.cbSize = sizeof(si);
		GetScrollInfo ( SB_VERT, &si, SIF_TRACKPOS );
		m_viewPos = si.nTrackPos*((m_width / 8) * m_planes * m_height * m_columns);
		break;
	}

	if ( m_viewPos < 0 )
		m_viewPos = 0;
	DWORD dwDataLen = p_doc->size/(m_digitsInData/2)/((m_width / 8) * m_planes * m_height * m_columns) * ((m_width / 8) * m_planes * m_height * m_columns);
	if ( DWORD(m_viewPos) > dwDataLen - ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy) )
		m_viewPos = dwDataLen - ((m_width / 8) * m_planes * m_height * m_columns)*(m_charCountWindow.cy);

	if ( m_viewPos != nPrevViewPos )
	{
		InvalidateRect(NULL, FALSE);
		SetScrollPos ( SB_VERT, m_viewPos / ((m_width / 8) * m_planes * m_height * m_columns) );
		PlaceCaret(true);
	}

//	CView::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CGfxEditView::PlaceCaret(bool check_focus)
{
	if(check_focus)
		if(!GetFocus())
			if(GetFocus()->m_hWnd == m_hWnd)	// does this window have focus?
				return;

	POINT pt;
	
	if( !IsSelectionEmpty() && m_editPos == m_selEnd+1 )
	{
		pt.x = (m_selEnd % ((m_width / 8) * m_planes * m_height * m_columns));
		pt.x /= ((m_width / 8) * m_planes * m_height);
		pt.y = (m_selEnd - m_viewPos)/((m_width / 8) * m_planes * m_height * m_columns);
	}
	else
	{
		pt.x  = (m_editPos - (m_viewPos));
		pt.x /= ((m_width / 8) * m_planes * m_height);
		pt.y  = pt.x / m_columns;
		pt.x %= m_columns;
	}
	m_fire_thing->SetData(&m_Bitmap, pt.x * m_width, pt.y * m_height, m_width, m_height);

	pt.x = ((pt.x - m_nHorzScroll) * m_width * 2) + m_cellSize.cx / 2;

	if ( m_showAddress )
		pt.x += (m_addrMargin * m_cellSize.cx);

	pt.y *= m_height * 2;
	SetCaretPos ( pt );
}

inline void CGfxEditView::ClearSelection()
{
	m_selStart = m_selEnd = -1;
	InvalidateRect(NULL, FALSE);
}

inline BOOL CGfxEditView::IsSelectionEmpty()
{
	return ( m_selStart == -1 );
}

void CGfxEditView::ChangeView()
{
	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

void CGfxEditView::StorePosition(void)
{
	SearchRConfig::store_m_dwStartAddr = m_dwStartAddr;
	SearchRConfig::store_m_nEditDigit = m_nEditDigit;
	SearchRConfig::store_m_selStart = m_selStart;
	SearchRConfig::store_m_selEnd = m_selEnd;
	SearchRConfig::store_m_editPos = m_editPos;
	SearchRConfig::store_m_viewPos = m_viewPos;
	SearchRConfig::store_m_nHorzScroll = m_nHorzScroll;
	SearchRConfig::store_m_editMode = m_editMode;
}

void CGfxEditView::RestorePosition(void)
{
	m_dwStartAddr = SearchRConfig::store_m_dwStartAddr;
	m_nEditDigit = SearchRConfig::store_m_nEditDigit;
	m_selStart = SearchRConfig::store_m_selStart;
	m_selEnd = SearchRConfig::store_m_selEnd;
	m_editPos = SearchRConfig::store_m_editPos;
	m_viewPos = SearchRConfig::store_m_viewPos;
	m_nHorzScroll = SearchRConfig::store_m_nHorzScroll;
	m_editMode = SearchRConfig::store_m_editMode;
}

void CGfxEditView::JumpToSelected(DWORD start, DWORD end)
{
	m_selStart = start;
	m_selEnd = end;
	m_editPos = start;
	m_viewPos = start;

	m_editPos /= ((m_width / 8) * m_planes * m_height);
	m_editPos *= ((m_width / 8) * m_planes * m_height);

	m_viewPos /= ((m_width / 8) * m_planes * m_height);
	m_viewPos *= ((m_width / 8) * m_planes * m_height);

	m_byte_pos = start - m_viewPos;

	m_byte_pos %= ((m_width / 8) * m_planes * m_height);

	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

void CGfxEditView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);

	if(!p_doc->data)
		return;

	DestroyCaret();	
}

void CGfxEditView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);

	if(!p_doc->data)
		return;

	CreateSolidCaret(m_width * 2, m_height * 2);
	PlaceCaret(true);
	if(!m_bMouseDown)
		ShowCaret();
}

void CGfxEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	BOOL bShift = ::GetKeyState(VK_SHIFT) & 0x80000000;
	BOOL bControl = ::GetKeyState(VK_CONTROL) & 0x80000000;

	if ( nChar == VK_TAB )
	{
		CWnd* pParent = GetParentOwner();
		if ( !pParent ) return;
		if ( bShift && !bControl )
		{
			::SendMessage(pParent->m_hWnd, WM_NEXTDLGCTL, 1, 0);
			return;
		}
		else if ( !bControl && !bShift )
		{
			::SendMessage(pParent->m_hWnd, WM_NEXTDLGCTL, 0, 0);
			return;
		}
	}
	
	if(!p_doc->data)
		return;

	if ( bShift && !bControl )
	{
		int prevEditPos = m_editPos;
		switch ( nChar )
			// SHIFT+arrows - modify selection
		{
		case VK_LEFT:
			ChangeEditPos ( -1, false);
			break;
		case VK_RIGHT:
			ChangeEditPos ( 1, false);
			break;
		case VK_UP:
			ChangeEditPos ( -1, false);
			break;
		case VK_DOWN:
			ChangeEditPos ( 1, false);
			break;
		case VK_PRIOR:
			ChangeEditPos ( 0, -m_charCountWindow.cy, false);
			break;
		case VK_NEXT:
			ChangeEditPos ( 0, m_charCountWindow.cy, false);
			break;
		case VK_HOME:
			m_nHorzScroll = 0;
			if ( bControl )
				// select to the start of data
			{
				m_editPos = 0;
				ChangeEditPos ( 0, 0, false);
			}
			else	// select to the beginning of the line
				ChangeEditPos ( -(m_editPos % ((m_width / 8) * m_planes * m_height * m_columns)),  0, false);
			break;
		case VK_END:
			if ( bControl )
				// select to the end of data
			{
				m_editPos = p_doc->size/(m_digitsInData/2);
				ChangeEditPos ( 0, 0, false);
			}
			else	// select to the end of the line
				ChangeEditPos ( ((m_width / 8) * m_planes * m_height * m_columns)-(m_editPos % ((m_width / 8) * m_planes * m_height * m_columns)),  0, false);
			break;
		case VK_INSERT:
			OnEditPaste();
			return;
		case VK_DELETE:
			OnEditCut();
			return;
		default:
			CView::OnKeyDown(nChar, nRepCnt, nFlags);
			return;
		}
		// recalc selection margins
		if ( IsSelectionEmpty() )
		{
			if ( m_editPos < prevEditPos )
			{
				m_selStart = m_editPos;
				m_selEnd = prevEditPos-1;
			}
			else if ( m_editPos > prevEditPos )
			{
				m_selStart = prevEditPos;
				m_selEnd = m_editPos-1;
			}
		}
		else
		{
			if ( prevEditPos-1 == m_selEnd )
				m_selEnd = m_editPos-1;
			else
				m_selStart = m_editPos;
			if ( m_selStart == m_selEnd+1 )
				ClearSelection();
		}
		InvalidateRect(NULL, FALSE);
	}
	else
	{
		if ( bControl && !bShift)
		{
			switch ( nChar )
			{
			case VK_HOME:
				m_editPos = 0;
				ChangeEditPos ( 0, 0, false);
				break;
			case VK_END:
				m_editPos = p_doc->size;
				ChangeEditPos ( 0, 0, false);
				break;
			case VK_INSERT:
				OnEditCopy();
				break;
			case 'A':
				OnEditSelectAll();
				break;
			case VK_TAB:	
				{	// switch hex/ascii
					ChangeEditPos(0,0, false);		// update scrollbars and caret pos
				}
				break;
			default:
				CView::OnKeyDown(nChar, nRepCnt, nFlags);
				break;
			}
		}
		else if ( !bControl && !bShift )
		{
			switch ( nChar )
			{
			case VK_LEFT:
				ChangeEditPos ( -1,  0, false);
				break;
			case VK_RIGHT:
				ChangeEditPos ( 1, 0, false);
				break;
			case VK_UP:
				ChangeEditPos ( 0, -1, false);
				break;
			case VK_DOWN:
				ChangeEditPos ( 0, 1, false);
				break;
			case VK_PRIOR:
				ChangeEditPos ( 0, -m_charCountWindow.cy, false);
				break;
			case VK_NEXT:
				ChangeEditPos ( 0, m_charCountWindow.cy, false);
				break;
			case VK_HOME:
				m_nHorzScroll = 0;
				ChangeEditPos ( -(m_editPos % ((m_width / 8) * m_planes * m_height * m_columns)),  0, false);
				break;
			case VK_END:	// todo???: place caret at the end
				ChangeEditPos ( ((m_width / 8) * m_planes * m_height * m_columns)-(m_editPos % ((m_width / 8) * m_planes * m_height * m_columns))-1,  0, false);
				break;
			case VK_BACK:
				if ( m_allowChangeSize )
				{
					if ( IsSelectionEmpty() )
					{
						if ( m_editPos > 0 )
						{
							ChangeEditPos ( -1,  0, true );
							DeleteData ( m_editPos );
							p_doc->SetModifiedFlag(TRUE);
						}
					}
					else
						DeleteSelection();
					InvalidateRect(NULL, FALSE);
					PlaceCaret(true);
				}
				break;
			case VK_DELETE:
				if ( m_allowChangeSize )
				{
					if ( IsSelectionEmpty() )
					{
						if ( (DWORD)m_editPos < p_doc->size )
						{
							DeleteData ( m_editPos );
							ChangeEditPos ( 0,  0, true );
							p_doc->SetModifiedFlag(TRUE);
						}
					}
					else
					{
						DeleteSelection();
						InvalidateRect(NULL, FALSE);
						PlaceCaret(true);
					}
				}
				else // if !EDIT_INSERT
				{
					if ( !IsSelectionEmpty() )
					{
						// zero selected bytes
						BYTE* buf;
						buf = p_doc->data;
						//m_data.AccessData((void**)&buf);
						memset ( buf+m_selStart*(m_digitsInData/2), 0, (m_selEnd-m_selStart+1)*(m_digitsInData/2));
						//m_data.UnaccessData();
						ClearSelection();
					}
				}
				break;
			case VK_INSERT:
				CreateSolidCaret (m_width * 2, m_height * 2);
				ShowCaret();
				PlaceCaret(true);
				break;
			default:
				CView::OnKeyDown(nChar, nRepCnt, nFlags);
				return;
			}

			if ( !IsSelectionEmpty() )
			{
				ClearSelection();
				InvalidateRect(NULL, FALSE);
			}
		}
	}
	
	//CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CGfxEditView::ChangeEditPos(long bytes, BOOL bRepaint)
{
	m_byte_pos += bytes;

	if((m_viewPos + m_byte_pos) < 0)
		m_byte_pos = 0 - m_viewPos;
	//else if((m_viewPos + m_byte_pos) < 0)
	//	m_byte_pos = 0 - m_viewPos;

	m_editPos /= ((m_width / 8) * m_planes * m_height);
	m_editPos *= ((m_width / 8) * m_planes * m_height);

	m_viewPos /= ((m_width / 8) * m_planes * m_height);
	m_viewPos *= ((m_width / 8) * m_planes * m_height);

	PlaceCaret(true);
}

void CGfxEditView::ChangeEditPos(long dx, long dy, BOOL bRepaint /* = false */)
{
	m_nEditDigit = 0;
	m_editPos += (dx + (dy * m_columns)) * ((m_width / 8) * m_planes * m_height);

	if ( NormalizeEditPos())
		bRepaint = true;

	if ( UpdateScrollBars())
		bRepaint = true;

	if ( bRepaint )
		InvalidateRect(NULL, FALSE);
	PlaceCaret(true);
}

void CGfxEditView::ChangeEditPos(CPoint pt)
{
	pt.x = (pt.x - ((m_addrMargin * m_cellSize.cx) + (m_cellSize.cx / 2))) / ((LONG) (m_width * 2));
	pt.y /= (m_height * 2);

	pt.x += m_nHorzScroll;

	if ( pt.x < 0 )
		pt.x = 0;
	else if ( pt.x >= m_columns)
		pt.x = m_columns - 1;

	m_editPos = m_viewPos;
	m_editPos += (pt.x + (pt.y * m_columns)) * ((m_width / 8) * m_planes * m_height);

	NormalizeEditPos();
	UpdateScrollBars();
}

void CGfxEditView::OnEditCut()
{
	if ( m_allowChangeSize )
	{
		if ( !IsSelectionEmpty() ) 
		{
			OnEditCopy();
			DeleteSelection();
			InvalidateRect(NULL, FALSE);
		}
	}
}

void CGfxEditView::OnEditCopy()
{
	if ( IsSelectionEmpty() ) return;

	if ( !OpenClipboard() )
		return;
	VERIFY(::EmptyClipboard());

		// size of selection in bytes
	int	dwLen = (m_selEnd-m_selStart+1)*(m_digitsInData/2);

		// alloc memory for "BinaryData" clipboard format
	HGLOBAL	hMemb = ::GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_ZEROINIT, dwLen+sizeof(DWORD));
		// alloc memory for CF_TEXT clipboard format
	HGLOBAL	hMema = ::GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_ZEROINIT, dwLen+1);
	
	ASSERT ( hMemb );
	ASSERT ( hMema );
	
	// copy binary
	BYTE* buf;
	LPBYTE	p = (BYTE*)::GlobalLock(hMemb);

		// first DWORD in BinaryData contains the length of data in bytes
	*(DWORD*) p = dwLen;
	p += sizeof(DWORD);

	buf = p_doc->data;
	memcpy(p, buf+m_selStart*(m_digitsInData/2), dwLen);

	::GlobalUnlock(hMemb);
	
	// copy text
	p = (BYTE*)::GlobalLock(hMema);
	memcpy(p, buf+m_selStart*(m_digitsInData/2), dwLen);
	p[dwLen] = 0;

	::GlobalUnlock(hMema);

	VERIFY(::SetClipboardData((USHORT)RegisterClipboardFormat(_T("BinaryData")), hMemb));
	VERIFY(::SetClipboardData(CF_TEXT, hMema));
	::CloseClipboard();
}

void CGfxEditView::OnEditPaste()
{
	BOOL bIsBinaryData = FALSE;
	HGLOBAL hmem = NULL;
	UINT hFormat = RegisterClipboardFormat(_T("BinaryData"));
	// we can paste CF_TEXT or BinaryData
	if (!::IsClipboardFormatAvailable((USHORT)hFormat) && !::IsClipboardFormatAvailable(CF_TEXT))
		return;
	
	VERIFY ( OpenClipboard() );
	if (::IsClipboardFormatAvailable((USHORT)hFormat))
	{
		hmem = ::GetClipboardData((USHORT)hFormat);
		bIsBinaryData = TRUE;
	}
	else if (::IsClipboardFormatAvailable(CF_TEXT))
	{
		hmem = ::GetClipboardData(CF_TEXT);
	}
	
	if(hmem)
	{
		LPBYTE	p = (BYTE*)::GlobalLock(hmem);
		DWORD	dwLen;
		if ( bIsBinaryData )
		{
			// first DWORD in BinaryData contains the length of data in bytes
			dwLen = *(DWORD*)p;
			p += sizeof(DWORD);
		}
		else
			dwLen = (DWORD) ::GlobalSize(hmem)-1;
		
		if ( dwLen > 0 )
		{
			{
				if ( !IsSelectionEmpty() )
				{
					m_editPos = m_selStart;
					if ( dwLen > ULONG(m_selEnd - m_selStart + 1) )
						dwLen = m_selEnd - m_selStart + 1;
					ClearSelection();
				}
				
				DWORD dwDataLen = p_doc->size;
				if ( m_editPos*(m_digitsInData/2)+dwLen > dwDataLen )
				{
					// need to add bytes at the end
					if (m_allowChangeSize)
						InsertData ( dwDataLen/(m_digitsInData/2), (dwLen - (dwDataLen-m_editPos*(m_digitsInData/2))+(m_digitsInData/2)-1)/(m_digitsInData/2) );
					else
						dwLen = dwDataLen - m_editPos*(m_digitsInData/2);
				}
			}
			
			if ( dwLen > 0 )
			{
				BYTE* buf;
				buf = p_doc->data;
				memcpy ( buf+m_editPos*(m_digitsInData/2), p, dwLen );
				p_doc->SetModifiedFlag(TRUE);
				InvalidateRect(NULL, FALSE);
			}
		}
		::GlobalUnlock(hmem);
	}
	::CloseClipboard();
}

void CGfxEditView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!IsSelectionEmpty());
}

void CGfxEditView::OnUpdateEditCut(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!IsSelectionEmpty() && (m_allowChangeSize));
}

void CGfxEditView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	//COleDataObject obj;
	//if (obj.AttachClipboard())
	//	pCmdUI->Enable(obj.IsDataAvailable(CF_TEXT) || obj.IsDataAvailable((USHORT)RegisterClipboardFormat(_T("BinaryData"))));
	//else
		pCmdUI->Enable(false);
}

void CGfxEditView::OnEditSelectAll()
{
	if ( p_doc->size == 0 ) 
		return;
	m_editPos = m_selStart = 0;
	m_selEnd = p_doc->size/(m_digitsInData/2)-1;
	ChangeEditPos (0, 0, true );	// place caret & repaint
}

void CGfxEditView::OnUpdateEditSelectAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(p_doc->size != 0);
}

void CGfxEditView::DeleteData(DWORD dwPos, DWORD dwCount /*=1*/)
{
	// convert to bytes
	dwPos *= (m_digitsInData/2);
	dwCount *= (m_digitsInData/2);

	DWORD dwArrSize = p_doc->size;
	ASSERT ( dwCount > 0 );
	ASSERT ( dwPos < dwArrSize );

	if ( dwPos != dwArrSize-1 )
	{
		BYTE* buf;
		buf = p_doc->data;
		memmove ( buf+dwPos, buf+dwPos+dwCount, dwArrSize-dwPos-dwCount );
	}

	p_doc->size = dwArrSize - dwCount;
	RecalcLayout();
}

void CGfxEditView::DeleteSelection()
{
	if ( !IsSelectionEmpty())
	{
		DeleteData ( m_selStart, m_selEnd-m_selStart+1 );
		m_editPos = m_selStart;
		p_doc->SetModifiedFlag(TRUE);
		ClearSelection();
		NormalizeEditPos();
		PlaceCaret(true);
	}
}

void CGfxEditView::InsertData(DWORD dwPos, DWORD dwCount/*=1*/)
{
	// convert to bytes
	dwPos *= (m_digitsInData/2);
	dwCount *= (m_digitsInData/2);

	DWORD dwArrSize = p_doc->size;
	ASSERT ( dwCount > 0 );
	ASSERT ( dwPos <= dwArrSize );

	if ( dwArrSize+dwCount <= dwArrSize || dwArrSize+dwCount >= 0x80000000 )
	{
		// overflow
		AfxThrowMemoryException ();
		return;
	}

	p_doc->size = dwArrSize + dwCount;

	if ( dwPos != dwArrSize )
	{
		BYTE* buf;
		buf = p_doc->data;

		memmove ( buf+dwPos+dwCount, buf+dwPos, dwArrSize-dwPos );
		memset ( buf+dwPos, 0, dwCount );
	}
	RecalcLayout();
}

void CGfxEditView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if(!p_doc->data)
		return;

	{
		if((nChar >= '0' && nChar <= '9') || (nChar >= 'a' && nChar <= 'f') || (nChar >= 'A' && nChar <= 'F'))
		{
			UINT b = nChar - '0';
			if(b > 9)
			{
				if (b <= 'F' - '0')
					b = 10 + nChar - 'A';
				else
					b = 10 + nChar - 'a';
			}
			UINT mask = 0xFUL << (( m_digitsInData - m_nEditDigit - 1 )*4);

			if ( !EnterNumber())
				return;

			BYTE* buf;
			DWORD bufAddr = m_editPos*(m_digitsInData/2);
			buf = p_doc->data;
			*(DWORD*)(buf+bufAddr) = *(DWORD*)(buf+bufAddr) & ~mask | (b << (( m_digitsInData - m_nEditDigit - 1 )*4));
			
			m_nEditDigit++;
			if (NormalizeEditPos())		
				InvalidateRect(NULL, FALSE);			// cursor was outside visible area
			InvalidateEditRect ( m_editPos );
			if ( m_nEditDigit == m_digitsInData )
				ChangeEditPos(1, 0, false);
			else
			{
				UpdateScrollBars();
				PlaceCaret(true);
			}
		}	
		else
			CView::OnChar(nChar, nRepCnt, nFlags);
	}
}

BOOL CGfxEditView::EnterNumber()
{
	if ( !IsSelectionEmpty())
	{
		if ( m_allowChangeSize )
		{
			// delete current selection and insert 1 number
			m_editPos = m_selStart;
			DeleteSelection();
			InsertData ( m_editPos, 1 );
			InvalidateRect(NULL, FALSE);
		}
		else
		{
			ClearSelection();
		}
	}
	else	// is m_editPos at the end?
		if ( DWORD(m_editPos)*(m_digitsInData/2) == p_doc->size)
		{
			if ( m_allowChangeSize )
			{
				InsertData ( m_editPos, 1 );
				if ( m_editPos % ((m_width / 8) * m_planes * m_height * m_columns) != ((m_width / 8) * m_planes * m_height * m_columns)-1)
					InvalidateEditRect (m_editPos);
				else
					InvalidateRect(NULL, FALSE);	// we need to add new line
			}
			else
				return false;
		}
		else if	(m_nEditDigit == 0 )
			// insert 1 number
		{
			InsertData ( m_editPos, 1 );
			InvalidateRect(NULL, FALSE);
		}
	p_doc->SetModifiedFlag(TRUE);
	return true;
}

void CGfxEditView::InvalidateEditRect(long nEditPos)
{
	CRect rcInvalid;
	long nPos = nEditPos - m_viewPos;
	ASSERT ( nPos >= 0 && nPos < ((m_width / 8) * m_planes * m_height * m_columns)*m_charCountWindow.cy );

	// invalidate rect of the number
	rcInvalid.left = (nPos % ((m_width / 8) * m_planes * m_height * m_columns) * (m_digitsInData+1) + m_addrMargin - m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2;
	rcInvalid.top = nPos / ((m_width / 8) * m_planes * m_height * m_columns) * m_cellSize.cy;
	rcInvalid.right = rcInvalid.left + (m_digitsInData+1)*m_cellSize.cx;
	rcInvalid.bottom = rcInvalid.top + m_cellSize.cy;
	InvalidateRect(rcInvalid);

	/*{
		// invalidate rect of corresponding ASCII char
		rcInvalid.left = (((nPos % ((m_width / 8) * m_planes * m_height * m_columns)) * CHAR_PADDING)+ m_asciiMargin - m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2;
		rcInvalid.top = nPos / ((m_width / 8) * m_planes * m_height * m_columns) * m_cellSize.cy;
		rcInvalid.right = rcInvalid.left + (m_cellSize.cx * CHAR_PADDING);
		rcInvalid.bottom = rcInvalid.top + m_cellSize.cy;
		InvalidateRect(rcInvalid);
	}*/
}

void CGfxEditView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	m_bMouseDown = true;

	if(!IsSelectionEmpty())
		ClearSelection();

	if(nFlags & MK_SHIFT)
	{
		if(DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
			m_posMouseDown = m_editPos - 1;
		else
			m_posMouseDown = m_editPos;
		RecalcSelection(point);
		m_bMouseMove = true;
	}
	else
	{
		ChangeEditPos(point);
		if(DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
			m_posMouseDown = m_editPos - 1;
		else
			m_posMouseDown = m_editPos;
	}
	SetCapture();
	HideCaret();
	CView::OnLButtonDown(nFlags, point);
}

void CGfxEditView::RecalcSelection(CPoint point)
{
	if ( p_doc->size == 0 )
		return;
	if ( m_bMouseDown )
	{
		int nPrevSelStart = m_selStart, nPrevSelEnd = m_selEnd;

		ChangeEditPos ( point );
		if ( m_posMouseDown < m_editPos )
		{
			if ( DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
				m_editPos--;
			m_selStart = m_posMouseDown; 
			m_selEnd = m_editPos;
			m_editPos = m_selEnd;
			m_nEditDigit = 0;
		}
		else
		{
			m_selStart = m_editPos;
			m_selEnd = m_posMouseDown; 
			m_editPos = m_selStart;
			m_nEditDigit = 0;
		}
		if ( nPrevSelStart != m_selStart || nPrevSelEnd != m_selEnd )
			InvalidateRect(NULL, FALSE);
	}
}

void CGfxEditView::OnLButtonUp(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	if ( m_bMouseDown )
	{
		if ( m_bMouseMove )
			RecalcSelection(point);
		ReleaseCapture();
		m_bMouseDown = false;
		m_bMouseMove = false;
		if ( m_bTimer )
		{
			KillTimer(ID_TIMER);
			m_bTimer = false;
		}

		// place caret just after the selection
		if ( !IsSelectionEmpty() )
		{
			if ( m_editPos == m_selEnd )
				m_editPos++;
			m_nEditDigit = 0;
		}
	}
	PlaceCaret(true);
	ShowCaret();
	CView::OnLButtonUp(nFlags, point);
}

void CGfxEditView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	ChangeEditPos(point);
	
	// select the number under cursor if it isn't at the end of data
	if ( DWORD(m_editPos*(m_digitsInData/2)) != p_doc->size)
	{
		m_selStart = m_selEnd = m_editPos;
		m_nEditDigit = 0;
		InvalidateEditRect(m_editPos);
		m_editPos++;
	}

	PlaceCaret(true);
	m_bMouseDown = false;
	ReleaseCapture();

	CView::OnLButtonDblClk(nFlags, point);
}

UINT CGfxEditView::OnGetDlgCode()
{
	return CView::OnGetDlgCode() | DLGC_WANTARROWS | DLGC_WANTCHARS | DLGC_WANTTAB;
}

void CGfxEditView::OnMouseMove(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	if ( m_bMouseDown )
	{
		m_bMouseMove = true;
		if ( !m_bTimer && !m_rcClient.PtInRect(point))
		{
			// mouse went outside the window, so we need to scroll even if mouse doesn't 
			// moving
			SetTimer ( ID_TIMER, 200, 0 );
			m_bTimer = true;
		}
		else if ( m_bTimer && m_rcClient.PtInRect(point))
		{
			// mouse is inside window, so we don't need the timer
			KillTimer ( ID_TIMER );
			m_bTimer = false;
		}
		m_prevMousePoint = point;
		RecalcSelection(point);
	}
	CView::OnMouseMove(nFlags, point);
}

void CGfxEditView::OnHScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/) 
{
	int nPrevHorzScroll = m_nHorzScroll;
	switch(nSBCode)
	{
	case SB_LINELEFT:
		m_nHorzScroll--;
		break;
	case SB_PAGELEFT:
		m_nHorzScroll -= m_charCountWindow.cx;
		break;
	case SB_LINERIGHT:
		m_nHorzScroll++;
		break;
	case SB_PAGERIGHT:
		m_nHorzScroll += m_charCountWindow.cx;
		break;
	case SB_THUMBTRACK:
		SCROLLINFO si;

		si.cbSize = sizeof(si);
		GetScrollInfo ( SB_HORZ, &si, SIF_TRACKPOS );
		m_nHorzScroll = si.nTrackPos;
		break;
	}
	if ( m_nHorzScroll < 0 )
		m_nHorzScroll = 0;
	else if ( m_nHorzScroll > (m_charCountView.cx - m_charCountWindow.cx) )
		m_nHorzScroll = (m_charCountView.cx - m_charCountWindow.cx);

	if ( nPrevHorzScroll != m_nHorzScroll )
	{
		InvalidateRect(NULL, FALSE);
		SetScrollPos ( SB_HORZ, m_nHorzScroll );
		PlaceCaret(true);
	}
}

void CGfxEditView::OnTimer(UINT nIDEvent)
{
	// called when mouse went outside window during selection
	ASSERT ( m_bTimer );

	if ( nIDEvent == ID_TIMER )
	{
		RecalcSelection ( m_prevMousePoint );
	}
	else
		CView::OnTimer(nIDEvent);
}

void CGfxEditView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	if(!p_doc->data)
		return;

	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

BOOL CGfxEditView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	m_viewPos += (-zDelta / 40) * ((m_width / 8) * m_planes * m_height * m_columns);

	if ( m_viewPos < 0)
	{
		m_viewPos = 0;
	}
	else if ( DWORD(m_viewPos) > p_doc->size / (m_digitsInData/2) - ((m_width / 8) * m_planes * m_height * m_columns)*m_charCountWindow.cy)
	{
		m_viewPos = p_doc->size / (m_digitsInData/2) / ((m_width / 8) * m_planes * m_height * m_columns) * ((m_width / 8) * m_planes * m_height * m_columns) - ((m_width / 8) * m_planes * m_height * m_columns)*m_charCountWindow.cy;
		if ( m_viewPos < 0 ) m_viewPos = 0;
	}

	UpdateScrollBars();
	InvalidateRect(NULL, FALSE);
	PlaceCaret(true);

	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CGfxEditView::SetColour(COLORREF colour, DWORD colour_def)
{
	m_colour_data[colour_def] = colour;
}


_TCHAR CGfxEditView::MakeChar(BYTE i)
{
	return i;
}


BYTE CGfxEditView::MakeByte(_TCHAR i)
{
	return (BYTE) i;
}


BOOL CGfxEditView::IsValidChar(BYTE i)
{
	return (_istprint((_TCHAR) i) != 0);
}

BOOL CGfxEditView::IsValidString(BYTE i)
{
	return FALSE;
}

void CGfxEditView::MakeString(BYTE i, CString& s)
{
	s = (char) i;
}

BOOL CGfxEditView::OnEraseBkgnd(CDC* pDC) 
{	
	CBrush backBrush(m_colour_data[BACK_COLOUR]);
	CRect rect;
	pDC->GetClipBox(&rect);
	pDC->FillRect(rect, &backBrush);
	return TRUE;
}

COLORREF gfx_edit_def_colours[CGfxEditView::COLOUR_COUNT] =
{
	RGB(0x00, 0x00, 0x00),
	RGB(0xFF, 0xFF, 0xFF),
	RGB(0x80, 0x80, 0x80),
	RGB(0x00, 0x00, 0x00),
	RGB(0x00, 0x00, 0xFF),
	RGB(0xFF, 0x00, 0x00),
};

void CGfxEditView::SetGfxMode(DWORD planes, DWORD width, DWORD height)
{
	m_planes = planes;
	m_width = width;
	m_height = height;
	m_columns = 256 / m_width;
	RecalcLayout();
	ChangeEditPos(0, 0, false);
	PlaceCaret(false);
	InvalidateRect(NULL, true);
}
