#pragma once

// CGfxEditView view

class CGfxEditView : public CBaseView
{
public:
	CGfxEditView();           // protected constructor used by dynamic creation
protected:
	DECLARE_DYNCREATE(CGfxEditView)

// Attributes
public:
	enum COLOUR_DEF {
		BACK_COLOUR,
		ADDRESS_COLOUR,
		HEX_COLOUR,
		SINGLE_CHAR_COLOUR,
		MULTIPLE_CHAR_COLOUR,
		UNKNOWN_CHAR_COLOUR,
		COLOUR_COUNT,
	};
	
	void SetColour(COLORREF colour, DWORD colour_def);

	// Convert BYTE to ASCII
	virtual _TCHAR MakeChar(BYTE i);

	// Convert ACSII to BYTE
	virtual BYTE MakeByte(_TCHAR i);

	// Check if BYTE to ASCII is valid
	virtual BOOL IsValidChar(BYTE i);

	virtual BOOL IsValidString(BYTE i);
	virtual void MakeString(BYTE i, CString& s);

	void JumpToSelected(DWORD start, DWORD end);

	virtual void ChangeView();

	void StorePosition(void);
	void RestorePosition(void);

public:
	void SetGfxMode(DWORD planes, DWORD width, DWORD height);

protected:
	COLORREF m_colour_data[COLOUR_COUNT];

	DWORD m_planes;
	DWORD m_width;
	DWORD m_height;
	DWORD m_columns;
	long m_byte_pos;

	/*! Data displayed.*/
	//COleSafeArray m_data;

	/*! Number of hex digits shown in address field (4 or 8).*/
	short m_digitsInAddress;

	/*! Number of hex digits in one number
	 - 2 - BYTE view
  	 - 4 - WORD view
	 - 8 - DWORD view
	 */
	short m_digitsInData;

	short m_font_height;

	/*! Starting address to display.*/
	DWORD m_dwStartAddr;

	/*! True if data are modified.*/
	BOOL m_bDataModified;

// Other data

	/*! This value equals the following expression: (m_showAscii && m_digitsInData == 2).*/
	BOOL m_bRealShowAscii;

	/*! Current horizontal scroll shift in chars.*/
	int m_nHorzScroll;

	/*! True after WM_LBUTTONDOWN, false after WM_LBUTTONUP.*/
	BOOL m_bMouseDown;

	/*! True after WM_MOUSEMOVE, false after WM_LBUTTONUP.*/
	BOOL m_bMouseMove;

	/*! Address of number under mouse cursor in WM_LBUTTONDOWN.*/
	int m_posMouseDown;

	/*! Window size in chars.*/
	CSize m_charCountWindow;

	/*! Total view size in chars.*/
	CSize m_charCountView;

	/*! Char size in device points.*/
	CSize m_cellSize;

	/* !!! IMPORTANT
		m_selStart, m_selEnd and other addresses of numbers are counts of numbers (NOT NESESSARY BYTES!),
		depending on m_digitsInData.*/

	/*! Selection start (in numbers).*/
	long m_selStart;

	/*! Selection end (in numbers).*/
	long m_selEnd;

	/*! Address of number being edited.*/
	long m_editPos;

	/*! Address of the first visible number.*/
	long m_viewPos;

	/*! Digit being edited (0 - leftmost).*/
	int m_nEditDigit;

	/*! Right margin of address field in chars.*/
	int m_addrMargin;

	/*! Left margin of ASCII field in chars.*/
	int m_asciiMargin;
	int m_tableMargin;

	/*! This value is a combination of following flags:
	- EDIT_BYTES - edit hex data mode (if not set - edit ASCII data)
	- EDIT_INSERT - insert mode (if not set - overwrite mode)
	*/
	int m_editMode;

	/*! Control client rect.*/
	CRect m_rcClient;

	/*! True if timer is active.*/
	BOOL m_bTimer;

	/*! Coordinates of last point in OnMouseMove.*/
	CPoint m_prevMousePoint;

	BOOL m_dataModified;
	BOOL m_showAddress;
	BOOL m_showAscii;
	BOOL m_allowChangeSize;

	void RecalcLayout();
	BOOL NormalizeEditPos();
	BOOL UpdateScrollBars();
	void PlaceCaret(bool check_focus);
	BOOL IsSelectionEmpty();
	void ClearSelection();
	void ChangeEditPos(long bytes, BOOL bRepaint);
	void ChangeEditPos(long dx, long dy, BOOL bRepaint);
	void ChangeEditPos(CPoint pt);
	void DeleteData ( DWORD dwPos, DWORD dwCount = 1 );
	void DeleteSelection();
	void InsertData(DWORD dwPos, DWORD dwCount/*=1*/);
	BOOL EnterNumber();
	void InvalidateEditRect(long nEditPos);
	void RecalcSelection(CPoint point);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGfxEditView)
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CGfxEditView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
protected:
	//{{AFX_MSG(CGfxEditView)
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnEditCopy();
	afx_msg void OnEditCut();
	afx_msg void OnEditPaste();
	afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditCut(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnEditSelectAll();
	afx_msg void OnUpdateEditSelectAll(CCmdUI* pCmdUI);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg UINT OnGetDlgCode();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
};

extern COLORREF gfx_edit_def_colours[CGfxEditView::COLOUR_COUNT];
