// GfxModeInfo.cpp : implementation file
//

#include "stdafx.h"
#include "searchr.h"
#include "TableDoc.h"
#include "SearchRDoc.h"
#include "GfxModeInfo.h"
#include "HexEditView.h"
#include "GfxEditView.h"
#include "GfxView.h"
#include "GfxModeAdd.h"

// CGfxMode object

class CGfxMode : public CObject
{
public:
	DWORD m_planes;
	DWORD m_width;
	DWORD m_height;
	CString m_name;
	CGfxMode()
	{
		m_planes = 1;
		m_width = 8;
		m_height = 8;
		m_name.Empty();
	}
#ifdef _DEBUG
	void Dump( CDumpContext& dc ) const
	{
		CObject::Dump( dc );
		dc << m_planes;
		dc << m_width;
		dc << m_height;
	}
#endif
};

// CGfxModeInfo dialog

IMPLEMENT_DYNAMIC(CGfxModeInfo, CDialog)

CGfxModeInfo::CGfxModeInfo(CWnd* pParent /*=NULL*/)
	: CDialog(CGfxModeInfo::IDD, pParent)
{
	ASSERT(pParent != NULL);

	m_pParent = pParent;
	m_nID = CGfxModeInfo::IDD;
}

CGfxModeInfo::~CGfxModeInfo()
{
}


void CGfxModeInfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_GFX_MODE_LIST, m_table_list);
	DDX_Control(pDX, IDC_GFX_MODE_ADD, m_table_add);
	DDX_Control(pDX, IDC_GFX_MODE_EDIT, m_table_edit);
	DDX_Control(pDX, ID_GFX_MODE_DELETE, m_table_delete);
}


BEGIN_MESSAGE_MAP(CGfxModeInfo, CDialog)
	//{{AFX_MSG_MAP(CGfxModeInfo)
	ON_BN_CLICKED(IDC_GFX_MODE_ADD, OnBnClickedGfxModeAdd)
	ON_BN_CLICKED(IDC_GFX_MODE_EDIT, OnBnClickedTableEdit)
	ON_BN_CLICKED(ID_GFX_MODE_DELETE, OnBnClickedTableDelete)
	ON_NOTIFY(NM_CLICK, IDC_GFX_MODE_LIST, OnNMClickTableList)
	ON_NOTIFY(NM_DBLCLK, IDC_GFX_MODE_LIST, OnDblclkGfxModeList)
	//}}AFX_MSG_MAP
	ON_WM_WINDOWPOSCHANGED()
END_MESSAGE_MAP()

// CGfxModeInfo message handlers

BOOL CGfxModeInfo::Create()
{
	return CDialog::Create(m_nID, m_pParent);
}

BOOL CGfxModeInfo::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	{
		CGfxMode *new_gfx_mode;
		
		int i,x,y;

		for(y = 8; y <= 32; y+=8)
		{
			for(x = 8; x <= 32; x+=8)
			{
				for(i = 0; i < 8; i ++)
				{
					new_gfx_mode = new CGfxMode;
					CString str_name;

					str_name.Format(_T("%dx%d %d plane(s)"), x, y, i + 1);
					new_gfx_mode->m_planes = i + 1;
					new_gfx_mode->m_width = x;
					new_gfx_mode->m_height = y;
					new_gfx_mode->m_name = str_name;

					m_gfx_mode_list.AddTail(new_gfx_mode);
				}
			}
		}
	}

	m_table_list.InsertColumn(0, _T("Name"), LVCFMT_LEFT, 128, -1);
	m_table_list.InsertColumn(1, _T("X"), LVCFMT_LEFT, 24, -1);
	m_table_list.InsertColumn(2, _T("Y"), LVCFMT_LEFT, 24, -1);
	m_table_list.InsertColumn(3, _T("P"), LVCFMT_LEFT, 24, -1);

	CImageList img2;
	CBitmap arse;

	arse.LoadBitmap(IDB_LIST_GFX_MODE);
	img2.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 0);
	img2.Add(&arse, RGB(0, 128, 128));
	m_table_list.SetImageList(&img2, LVSIL_SMALL);
	img2.Detach();
	arse.Detach();

	UpdateList();
	ChangeView();

	if(SearchRConfig::winpos_gfx_mode_info.Width() > 0)
		// restore the old window pos
		MoveWindow(SearchRConfig::winpos_gfx_mode_info, false);
	else
	{
		// centre in the parent window
		CRect r_parent, r_child;
		GetParent()->GetWindowRect(r_parent);
		GetWindowRect(r_child);
		MoveWindow(r_parent.right - r_child.Width() - 64, r_parent.top + 128, r_child.Width(), r_child.Height(), false);
	}

	return TRUE;
}

void CGfxModeInfo::ChangeView(void)
{
	m_table_edit.EnableWindow(m_table_list.GetSelectedCount());
	m_table_delete.EnableWindow(m_table_list.GetSelectedCount());
}

void CGfxModeInfo::UpdateList(void)
{
	m_table_list.DeleteAllItems();

	CGfxMode* curr_gfx_mode;

	CString str_bookmark;

	DWORD i;
	POSITION pos;

	i = 0;
	pos = m_gfx_mode_list.GetHeadPosition();

	while(pos != NULL)
	{
		curr_gfx_mode = (CGfxMode *) m_gfx_mode_list.GetAt(pos);

		m_table_list.InsertItem(i, curr_gfx_mode->m_name);
		str_bookmark.Format(_T("%d"), curr_gfx_mode->m_width);
		m_table_list.SetItemText(i, 1, str_bookmark);
		str_bookmark.Format(_T("%d"), curr_gfx_mode->m_height);
		m_table_list.SetItemText(i, 2, str_bookmark);
		str_bookmark.Format(_T("%d"), curr_gfx_mode->m_planes);
		m_table_list.SetItemText(i, 3, str_bookmark);
		m_table_list.SetItemData(i, (DWORD_PTR) curr_gfx_mode);
		i ++;
		m_gfx_mode_list.GetNext(pos);
	}
}

void CGfxModeInfo::OnBnClickedGfxModeAdd()
{
	CGfxModeAdd p_gfx_mode_add;

	if(p_gfx_mode_add.DoModal() == IDOK)
	{
		{
			CGfxMode *new_gfx_mode = new CGfxMode;
			CString str_name;

			str_name.Format(_T("%dx%d %d plane(s)"), p_gfx_mode_add.m_tile_width, p_gfx_mode_add.m_tile_height, p_gfx_mode_add.m_tile_planes);
			new_gfx_mode->m_planes = p_gfx_mode_add.m_tile_planes;
			new_gfx_mode->m_width = p_gfx_mode_add.m_tile_width;
			new_gfx_mode->m_height = p_gfx_mode_add.m_tile_height;
			new_gfx_mode->m_name = str_name;

			m_gfx_mode_list.AddTail(new_gfx_mode);
		}
	}

	UpdateList();
	ChangeView();
}

void CGfxModeInfo::OnBnClickedTableEdit()
{
	POSITION pos;
	
	pos = m_table_list.GetFirstSelectedItemPosition();
	DWORD i = m_table_list.GetNextSelectedItem(pos);

	CGfxModeAdd p_gfx_mode_add;

//	p_gfx_mode_add.m_add_offset.Format(_T("%02X"), m_table_list.GetItemData(i));
//	p_gfx_mode_add.m_add_data = m_table_list.GetItemText(i, 1);

	if(p_gfx_mode_add.DoModal() == IDOK)
	{
		/*if((p_gfx_mode_add.m_add_offset.IsEmpty() == FALSE) && (p_gfx_mode_add.m_add_data.IsEmpty() == FALSE))
		{
			if(p_table_doc)
			{
				DWORD j;
				_stscanf(p_gfx_mode_add.m_add_offset, _T("%X"), &j);
				p_table_doc->DeleteFromTable(j);
				p_table_doc->AddToTable(j, p_gfx_mode_add.m_add_data);
			}
		}*/
	}

	UpdateList();
	ChangeView();
}

void CGfxModeInfo::OnBnClickedTableDelete()
{
	POSITION pos;
	
	while(pos = m_table_list.GetFirstSelectedItemPosition())
	{
		int i = m_table_list.GetNextSelectedItem(pos);
		p_table_doc->DeleteFromTable(m_table_list.GetItemData(i));
		m_table_list.DeleteItem(i);
	}

	UpdateList();
	ChangeView();
}

void CGfxModeInfo::OnNMClickTableList(NMHDR *pNMHDR, LRESULT *pResult)
{
	ChangeView();
	*pResult = 0;
}

void CGfxModeInfo::OnDblclkGfxModeList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	POSITION pos;
	
	if(pos = m_table_list.GetFirstSelectedItemPosition())
	{
		CGfxMode* curr_gfx_mode;

		curr_gfx_mode = (CGfxMode *) m_table_list.GetItemData(m_table_list.GetNextSelectedItem(pos));

		p_view_4->SetGfxMode(curr_gfx_mode->m_planes, curr_gfx_mode->m_width, curr_gfx_mode->m_height);
		p_view_4->ChangeView();
	}
	
	*pResult = 0;
}

void CGfxModeInfo::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CDialog::OnWindowPosChanged(lpwndpos);
	// store the new window pos
	GetWindowRect(SearchRConfig::winpos_gfx_mode_info);
}
