// HexEditView.cpp : implementation file
//

#include "stdafx.h"
#include "searchr.h"
#include "HexEditView.h"
#include "TableDoc.h"
#include "SearchRDoc.h"


enum EDIT_MODE			// m_editMode flags
{
	EDIT_BYTES = 1,
	EDIT_INSERT = 2,
};

const ID_TIMER = 1;

#define CHAR_PADDING 2

/////////////////////////////////////////////////////////////////////////////
// CHexEditView

IMPLEMENT_DYNCREATE(CHexEditView, CView)

CHexEditView::CHexEditView()
{
	m_dwStartAddr = 0;
	m_nEditDigit = 0;
	m_selStart = m_selEnd = -1;
	m_editPos = m_viewPos = m_nHorzScroll = 0;
	m_editMode = EDIT_BYTES;

	m_bMouseDown = m_bMouseMove = FALSE;
	m_bTimer = FALSE;

	m_showAddress = TRUE;
	m_showAscii = TRUE;
	m_allowChangeSize = FALSE;
	m_digitsInAddress = 8;
	m_digitsInData = 2;
	m_columns = 16;
	m_fontHeight = 16;

	int i;
	for(i = 0; i < COLOUR_COUNT; i ++)
		m_colour_data[i] = hex_edit_def_colours[i];

	RecalcLayout();
}

CHexEditView::~CHexEditView()
{
}


BEGIN_MESSAGE_MAP(CHexEditView, CView)
	//{{AFX_MSG_MAP(CHexEditView)
	ON_WM_VSCROLL()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SELECT_ALL, OnUpdateEditSelectAll)
	ON_WM_CHAR()
	ON_WM_LBUTTONDOWN()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_GETDLGCODE()
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHexEditView drawing

void CHexEditView::OnDraw(CDC* pdc)
{
	CRect rc;
	GetClientRect(rc);

	CDC	dc;
	dc.CreateCompatibleDC(pdc);

	CBitmap bm;
	bm.CreateCompatibleBitmap(pdc, rc.Width(), rc.Height());
	dc.SelectObject(bm);

	if(!p_doc->data)
	{
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc, &bkBrush);
		pdc->BitBlt(0, 0, rc.Width(), rc.Height(), &dc, 0, 0, SRCCOPY);
		return;
	}


	CFont font;
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, 0 );
	font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("MS Gothic") );
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Januschan JOME Regular") );
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Arial") );

	CFont* oldFont = dc.SelectObject ( &font );

	dc.SetBoundsRect(&rc, DCB_DISABLE);

	CString strText;
	int nAddr = m_viewPos*(m_digitsInData/2);
	BYTE* buf;
	bool skip_byte;

	{
		CRect rc2 ( -(m_nHorzScroll*m_cellSize.cx), rc.top, (m_cellSize.cx/2)-(m_nHorzScroll*m_cellSize.cx), rc.bottom );
		CBrush bkBrush(m_colour_data[BACK_COLOUR]);
		dc.FillRect(rc2, &bkBrush);
	}
	{
		CRect rc2 ( (m_cellSize.cx/2) - (m_nHorzScroll * m_cellSize.cx), m_charCountWindow.cy*m_cellSize.cy, rc.right, rc.bottom);
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc2, &bkBrush);
	}
	{
		CRect rc2 ( (m_cellSize.cx/2) + ((m_charCountView.cx - m_nHorzScroll) * m_cellSize.cx), rc.top, rc.right, m_charCountWindow.cy*m_cellSize.cy);
		CBrush bkBrush((m_colour_data[BACK_COLOUR]));
		dc.FillRect(rc2, &bkBrush);
	}

	buf = p_doc->data;

	skip_byte = false;

	for ( int i = 0; i < m_charCountWindow.cy; i++ )
	{
		dc.SetTextColor((m_colour_data[HEX_COLOUR]));
		dc.SetBkColor((m_colour_data[BACK_COLOUR]));

		strText = ' ';
		dc.TextOut( (m_asciiMargin-m_nHorzScroll-1)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
		dc.TextOut( (m_charCountView.cx-m_nHorzScroll-1)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );

		if ( m_showAddress )
		{
			// draw address field
			if (DWORD(nAddr) < p_doc->size)
				strText.Format ( _T("%0*X  "), m_digitsInAddress, nAddr+m_dwStartAddr );
			else
				strText.Format ( _T("%*c  "), m_digitsInAddress, ' ' );
			dc.SetTextColor(m_colour_data[ADDRESS_COLOUR]);
			dc.SetBkColor(m_colour_data[BACK_COLOUR]);
			dc.TextOut( -m_nHorzScroll*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
		}
		int j, jj;
		for(j = 0, jj = 0; j < m_columns; j++ )
		{
			// draw hex data
			int dwPos = j+i*m_columns+m_viewPos;
			if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
			{
				dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
				dc.SetBkColor(GetSysColor(COLOR_HIGHLIGHT));
			}
			else
			{
				dc.SetTextColor(m_colour_data[HEX_COLOUR]);
				dc.SetBkColor(m_colour_data[BACK_COLOUR]);
			}
			
			if ( DWORD(dwPos) < p_doc->size/(m_digitsInData/2))
			{
				strText.Format ( _T("%02X "), buf[dwPos]);
				dc.TextOut( (m_addrMargin+j*(m_digitsInData+1)-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
				if(skip_byte == false)
				{
					if(MakeString(buf[dwPos], strText, 1))
					{
						if(strText.GetLength() == 1)
						{
							// 1 byte = single character

							if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
								dc.SetTextColor(m_colour_data[SINGLE_CHAR_COLOUR] ^ 0xFFFFFF);
							else
								dc.SetTextColor(m_colour_data[SINGLE_CHAR_COLOUR]);
						}
						else
						{
							// 1 byte = multiple characters (DTE/MTE)

							if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
								dc.SetTextColor(m_colour_data[MULTIPLE_CHAR_COLOUR] ^ 0xFFFFFF);
							else
								dc.SetTextColor(m_colour_data[MULTIPLE_CHAR_COLOUR]);
						}

						if(strText.IsEmpty())
						{
							if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
								dc.SetTextColor(m_colour_data[END_OF_STRING_COLOUR] ^ 0xFFFFFF);
							else
								dc.SetTextColor(m_colour_data[END_OF_STRING_COLOUR]);
							strText = _T("#");
						}
					}
					else
					{
						if(MakeString((buf[dwPos] << 8) | buf[dwPos + 1], strText, 2))
						{
							if(strText.GetLength() == 1)
							{
								// 2 bytes = single character

								if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
									dc.SetTextColor(m_colour_data[SINGLE_2_BYTE_CHAR_COLOUR] ^ 0xFFFFFF);
								else
									dc.SetTextColor(m_colour_data[SINGLE_2_BYTE_CHAR_COLOUR]);
							}
							else
							{
								// 2 bytes = multiple characters (DTE/MTE)

								if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
									dc.SetTextColor(m_colour_data[MULTIPLE_2_BYTE_CHAR_COLOUR] ^ 0xFFFFFF);
								else
									dc.SetTextColor(m_colour_data[MULTIPLE_2_BYTE_CHAR_COLOUR]);
							}

							skip_byte = true;
						}
						else
						{
							// 1 byte = unknown data

							if((dwPos >= m_selStart) && (dwPos <= m_selEnd))
								dc.SetTextColor(m_colour_data[UNKNOWN_CHAR_COLOUR] ^ 0xFFFFFF);
							else
								dc.SetTextColor(m_colour_data[UNKNOWN_CHAR_COLOUR]);

							strText = (_TCHAR) '.';
						}
					}

					for(int g = 0; g < strText.GetLength(); g++)
					{
						CString s;

						s = strText[g];

						dc.TextOut( (m_asciiMargin+jj-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, s);
						if(strText[g] < 0x100)
						{
							if(m_half_size_char[strText[g]])
								dc.TextOut( (m_asciiMargin+jj+1-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, _T(" "));
						}
						jj += 2;
					}
				}
				else
				{
					skip_byte = false;
				}
			}
			else
			{
				// if the last line has less than m_columns numbers, fill it with spaces
				dc.SetBkColor((m_colour_data[BACK_COLOUR]));
				strText.Format ( _T("%*c "), m_digitsInData, ' ');
				dc.TextOut( (m_addrMargin+j*(m_digitsInData+1)-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
				if ( m_bRealShowAscii )
				{
					ASSERT ( m_digitsInData == 2 );
					strText = _T("  ");
					dc.TextOut( (m_asciiMargin+jj-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
					jj += 2;
				}
			}
			if(jj < 32)
			{
				dc.SetBkColor((m_colour_data[BACK_COLOUR]));
				strText.Format ( _T("%*c"), 32 - jj, ' ');
				dc.TextOut( (m_asciiMargin+jj-m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2, i*m_cellSize.cy, strText );
			}
		}

		nAddr += m_columns*(m_digitsInData/2);
	}

	pdc->BitBlt(0, 0, rc.Width(), rc.Height(), &dc, 0, 0, SRCCOPY);
}


void CHexEditView::RecalcLayout()
{
	if ( m_hWnd == 0 ) return;		// invisible window

	CFont font;
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, 0 );
	font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("MS Gothic") );
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Januschan JOME Regular") );
	//font.CreateFont ( m_fontHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("Arial") );

	CDC* pdc = GetDC();
	ASSERT ( pdc );

	CFont* oldFont = pdc->SelectObject(&font);

	// determine char size
	m_cellSize = pdc->GetTextExtent(_T("0"));
	pdc->LPtoDP ( &m_cellSize );

	int i;
	for(i = 0; i < 0x100; i ++)
	{
		CString j;
		CSize k;

		j = (_TCHAR) i;
		k = pdc->GetTextExtent(j);
		pdc->LPtoDP(&k);
		if(k.cx <= m_cellSize.cx)
			m_half_size_char[i] = true;
		else
			m_half_size_char[i] = false;
	}

	pdc->SelectObject ( oldFont );
	ReleaseDC ( pdc );

	GetClientRect ( m_rcClient );

	// compute entire rect of the control, including scroll bars
	if (GetStyle() & WS_VSCROLL)
		m_rcClient.right += ::GetSystemMetrics(SM_CXVSCROLL);
	if (GetStyle() & WS_HSCROLL)
		m_rcClient.bottom += ::GetSystemMetrics(SM_CYHSCROLL);

	// before calculations
	BOOL bShowSbVert = false;	// need to show vertical scroll bar?
	BOOL bShowSbHorz = false;	// need to show horizontal scroll bar?

	// after calculations
	BOOL bShowSbVert2 = false;	
	BOOL bShowSbHorz2 = false;	

	m_bRealShowAscii = (m_digitsInData == 2 && m_showAscii);

	do
	{
		bShowSbVert = bShowSbVert2;
		bShowSbHorz = bShowSbHorz2;

		// first pass - compute params when scrollbars are hidden
		// next passes (0, 1, or 2) - recalulate params if client rect was shrunk after showing scrollbars
		m_charCountWindow.cx = (m_rcClient.Width() - m_cellSize.cx/2) / m_cellSize.cx;
		m_charCountWindow.cy = m_rcClient.Height() / m_cellSize.cy;

		m_charCountView.cx = 0;
		m_columns = 16;

		if (m_showAddress)
			m_charCountView.cx += m_digitsInAddress + 2;

		m_addrMargin = m_charCountView.cx;

		m_charCountView.cx += (m_columns * (m_digitsInData + 1)) + 1;

		m_asciiMargin = m_charCountView.cx;

		if(m_bRealShowAscii)
			m_charCountView.cx += (m_columns * (m_digitsInData/2) * CHAR_PADDING) + 1;

		m_charCountView.cy = p_doc->size / (m_columns*(m_digitsInData/2)) + 1;

		if ( m_charCountView.cx > m_charCountWindow.cx && !bShowSbHorz )
		{
			m_rcClient.bottom -= ::GetSystemMetrics(SM_CYHSCROLL);
			bShowSbHorz2 = true;
		}
		if ( m_charCountView.cy > m_charCountWindow.cy && !bShowSbVert )
		{
			m_rcClient.right -= ::GetSystemMetrics(SM_CXVSCROLL);
			bShowSbVert2 = true;
		}
	} while ((bShowSbHorz != bShowSbHorz2) || (bShowSbVert != bShowSbVert2));

//	m_nHorzScroll = 0;
	m_viewPos = m_viewPos / m_columns * m_columns;
	NormalizeEditPos();
	UpdateScrollBars();

	if (!(GetStyle() & WS_VSCROLL))
		m_viewPos = 0;

	if(!p_doc->data)
		return;

	CreateSolidCaret((m_editMode & EDIT_INSERT) ? 0 : m_cellSize.cx, m_cellSize.cy);
	PlaceCaret();
	if(!m_bMouseDown)
		ShowCaret();
}

BOOL CHexEditView::NormalizeEditPos()
{
	BOOL bRepaint = false;
	DWORD dwDataSize = p_doc->size/(m_digitsInData/2);

	if ( m_editPos < 0 )
		m_editPos = 0;
	else if ( DWORD(m_editPos) >= dwDataSize )
	{
		m_editPos = dwDataSize - 1;
		m_nEditDigit = 0;
	}

	// change m_viewPos if m_editPos is moved out of the visible area
	if ( m_viewPos > m_editPos )
	{
		m_viewPos = (m_editPos / m_columns) * m_columns;
		bRepaint = true;
	}
	else if ((m_viewPos + (m_columns * (m_charCountWindow.cy - 1))) <= m_editPos)
	{
		m_viewPos = ((m_editPos / m_columns) - (m_charCountWindow.cy - 1)) * m_columns;
		bRepaint = true;
	}

	// if m_viewPos is too near to the end of data, decrease m_viewPos so entire page is displayed
	// (this may happen after deleting large selection or changing properties)
	if ( DWORD(m_viewPos) > p_doc->size / (m_digitsInData/2) - m_columns*(m_charCountWindow.cy))
	{
		m_viewPos = p_doc->size / (m_digitsInData/2) / m_columns * m_columns - m_columns*(m_charCountWindow.cy);
		if ( m_viewPos < 0 ) m_viewPos = 0;
	}

	int nPosX;		// x coordinate of caret in view
	if ( m_editMode & EDIT_BYTES )
		nPosX = m_addrMargin + m_editPos % m_columns * (m_digitsInData+1);
	else	// EDIT_ASCII
		nPosX = m_asciiMargin + m_editPos % m_columns;
	
	if ( nPosX < m_nHorzScroll )
	{
		m_nHorzScroll = nPosX;
		bRepaint = true;
	}
	else
	{
		if ( m_editMode & EDIT_BYTES ) 
			nPosX += m_digitsInData;	// to make entire number visible
		else
			nPosX++;
		if ( nPosX-m_nHorzScroll >= m_charCountWindow.cx )
		{
			m_nHorzScroll = nPosX - m_charCountWindow.cx;
			bRepaint = true;
		}
	}
	return bRepaint;
}

BOOL CHexEditView::UpdateScrollBars()
{
	BOOL bRepaint = false;
	SCROLLINFO si, prevSI;

	si.cbSize = prevSI.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	si.nMin = 0;
	si.nMax = m_charCountView.cy - 1;
	si.nPage = m_charCountWindow.cy;
	si.nPos = (m_viewPos + m_columns - 1) / m_columns;

	if ( !bRepaint && ( !GetScrollInfo(SB_VERT, &prevSI, SIF_POS ) || si.nPos != prevSI.nPos ))
		// if scrollbar is about to show or its position changed, the control needs repainting
		bRepaint = true;

	if(si.nMax >= int(si.nPage))
	{
		ShowScrollBar(SB_VERT);
	}
	else
	{
		ShowScrollBar(SB_VERT, FALSE);
	}
	VERIFY(SetScrollInfo(SB_VERT, &si, TRUE));

	si.fMask = SIF_ALL;
	si.nMin = 0;
	si.nMax = m_charCountView.cx - 1;
	si.nPage = m_charCountWindow.cx;
	si.nPos = m_nHorzScroll = 0;

	if ( (si.nMax > (int)si.nPage) && ((si.nPos+int(si.nPage)) > si.nMax) )
		si.nPos = m_nHorzScroll = 0; //(si.nMax - si.nPage) - 1;
	if ( !bRepaint && ( !GetScrollInfo(SB_HORZ, &prevSI, SIF_POS ) || si.nPos != prevSI.nPos ))
		// if scrollbar is about to show or its position changed, the control needs repainting
		bRepaint = true;

	if(si.nMax >= (int)si.nPage )
	{
		ShowScrollBar(SB_HORZ);
	}
	else
	{
		ShowScrollBar(SB_HORZ, FALSE);
	}

	VERIFY(SetScrollInfo(SB_HORZ, &si, TRUE));
	return bRepaint;
}

/////////////////////////////////////////////////////////////////////////////
// CHexEditView diagnostics

#ifdef _DEBUG
void CHexEditView::AssertValid() const
{
	CView::AssertValid();
}

void CHexEditView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHexEditView message handlers

void CHexEditView::OnVScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/) 
{
	int nPrevViewPos = m_viewPos;

	switch(nSBCode)
	{
	case SB_LINEDOWN:
		m_viewPos += m_columns;
		break;
	case SB_LINEUP:
		m_viewPos -= m_columns;
		break;
	case SB_PAGEDOWN:
		m_viewPos += m_columns*m_charCountWindow.cy;
		break;
	case SB_PAGEUP:
		m_viewPos -= m_columns*m_charCountWindow.cy;
		break;
	case SB_THUMBTRACK:
		SCROLLINFO si;

		si.cbSize = sizeof(si);
		GetScrollInfo ( SB_VERT, &si, SIF_TRACKPOS );
		m_viewPos = si.nTrackPos*m_columns;
		break;
	}

	if ( m_viewPos < 0 )
		m_viewPos = 0;
	DWORD dwDataLen = p_doc->size/(m_digitsInData/2)/m_columns * m_columns;
	if ( DWORD(m_viewPos) > dwDataLen - m_columns*(m_charCountWindow.cy) )
		m_viewPos = dwDataLen - m_columns*(m_charCountWindow.cy);

	if ( m_viewPos != nPrevViewPos )
	{
		InvalidateRect(NULL, FALSE);
		SetScrollPos ( SB_VERT, m_viewPos / m_columns );
		PlaceCaret();
	}

//	CView::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CHexEditView::PlaceCaret()
{
	if(!GetFocus())
		return;

	if ( GetFocus()->m_hWnd == m_hWnd )	// does this window have focus?
	{
		POINT pt;
		
		if ( !IsSelectionEmpty() && m_editPos == m_selEnd+1 )
		{
			// place caret just after the selection
			if ( m_editMode & EDIT_BYTES )
			{
				pt.x = m_selEnd % m_columns * (m_digitsInData+1) + m_digitsInData + 1;
				if ( m_showAddress )
					pt.x += m_addrMargin;
			}
			else
				pt.x = m_asciiMargin + ((m_selEnd % m_columns + 1) * CHAR_PADDING);
			
			pt.y = (m_selEnd - m_viewPos)/m_columns;
		}
		else
		{
			if ( m_editMode & EDIT_BYTES )
			{
				pt.x = m_editPos % m_columns * (m_digitsInData+1) + m_nEditDigit;
				if ( m_showAddress )
					pt.x += m_addrMargin;
			}
			else
				pt.x = m_asciiMargin + ((m_editPos % m_columns) * CHAR_PADDING);
			
			pt.y = (m_editPos - m_viewPos)/m_columns;
		}
		pt.x = ((pt.x - m_nHorzScroll) * m_cellSize.cx) + (m_cellSize.cx / 2);

		pt.y *= m_cellSize.cy;
		SetCaretPos ( pt );
	}
}

inline void CHexEditView::ClearSelection()
{
	m_selStart = m_selEnd = -1;
	InvalidateRect(NULL, FALSE);
}

inline BOOL CHexEditView::IsSelectionEmpty()
{
	return ( m_selStart == -1 );
}

void CHexEditView::ChangeView()
{
	/*m_dwStartAddr = 0;
	m_nEditDigit = 0;
	m_selStart = m_selEnd = -1;
	m_editPos = m_viewPos = m_nHorzScroll = 0;
	m_editMode = EDIT_BYTES;*/

	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

void CHexEditView::StorePosition(void)
{
	SearchRConfig::store_m_dwStartAddr = m_dwStartAddr;
	SearchRConfig::store_m_nEditDigit = m_nEditDigit;
	SearchRConfig::store_m_selStart = m_selStart;
	SearchRConfig::store_m_selEnd = m_selEnd;
	SearchRConfig::store_m_editPos = m_editPos;
	SearchRConfig::store_m_viewPos = m_viewPos;
	SearchRConfig::store_m_nHorzScroll = m_nHorzScroll;
	SearchRConfig::store_m_editMode = m_editMode;
}

void CHexEditView::RestorePosition(void)
{
	m_dwStartAddr = SearchRConfig::store_m_dwStartAddr;
	m_nEditDigit = SearchRConfig::store_m_nEditDigit;
	m_selStart = SearchRConfig::store_m_selStart;
	m_selEnd = SearchRConfig::store_m_selEnd;
	m_editPos = SearchRConfig::store_m_editPos;
	m_viewPos = SearchRConfig::store_m_viewPos;
	m_nHorzScroll = SearchRConfig::store_m_nHorzScroll;
	m_editMode = SearchRConfig::store_m_editMode;
}

void CHexEditView::JumpToSelected(DWORD start, DWORD end)
{
	m_selStart = start;
	m_selEnd = end;
	m_editPos = start;
	m_viewPos = start;

	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

void CHexEditView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);

	if(!p_doc->data)
		return;

	DestroyCaret();	
}

void CHexEditView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);

	if(!p_doc->data)
		return;

	CreateSolidCaret((m_editMode & EDIT_INSERT) ? 0 : m_cellSize.cx, m_cellSize.cy );
	PlaceCaret();
	if(!m_bMouseDown)
		ShowCaret();
}

void CHexEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	BOOL bShift = ::GetKeyState(VK_SHIFT) & 0x80000000;
	BOOL bControl = ::GetKeyState(VK_CONTROL) & 0x80000000;

	if ( nChar == VK_TAB )
	{
		CWnd* pParent = GetParentOwner();
		if ( !pParent ) return;
		if ( bShift && !bControl )
		{
			::SendMessage(pParent->m_hWnd, WM_NEXTDLGCTL, 1, 0);
			return;
		}
		else if ( !bControl && !bShift )
		{
			::SendMessage(pParent->m_hWnd, WM_NEXTDLGCTL, 0, 0);
			return;
		}
	}
	
	if(!p_doc->data)
		return;

	if ( bShift && !bControl )
	{
		int prevEditPos = m_editPos;
		switch ( nChar )
			// SHIFT+arrows - modify selection
		{
		case VK_LEFT:
			ChangeEditPos ( -1,  0, false);
			break;
		case VK_RIGHT:
			ChangeEditPos ( 1, 0, false);
			break;
		case VK_UP:
			ChangeEditPos ( 0, -1, false);
			break;
		case VK_DOWN:
			ChangeEditPos ( 0, 1, false);
			break;
		case VK_PRIOR:
			ChangeEditPos ( 0, -m_charCountWindow.cy, false);
			break;
		case VK_NEXT:
			ChangeEditPos ( 0, m_charCountWindow.cy, false);
			break;
		case VK_HOME:
			if ( m_editMode & EDIT_BYTES )
				m_nHorzScroll = 0;
			if ( bControl )
				// select to the start of data
			{
				m_editPos = 0;
				ChangeEditPos ( 0, 0, false);
			}
			else	// select to the beginning of the line
				ChangeEditPos ( -(m_editPos % m_columns),  0, false);
			break;
		case VK_END:
			if ( bControl )
				// select to the end of data
			{
				m_editPos = p_doc->size/(m_digitsInData/2);
				ChangeEditPos ( 0, 0, false);
			}
			else	// select to the end of the line
				ChangeEditPos ( m_columns-(m_editPos % m_columns),  0, false);
			break;
		case VK_INSERT:
			OnEditPaste();
			return;
		case VK_DELETE:
			OnEditCut();
			return;
		default:
			CView::OnKeyDown(nChar, nRepCnt, nFlags);
			return;
		}
		// recalc selection margins
		if ( IsSelectionEmpty() )
		{
			if ( m_editPos < prevEditPos )
			{
				m_selStart = m_editPos;
				m_selEnd = prevEditPos-1;
			}
			else if ( m_editPos > prevEditPos )
			{
				m_selStart = prevEditPos;
				m_selEnd = m_editPos-1;
			}
		}
		else
		{
			if ( prevEditPos-1 == m_selEnd )
				m_selEnd = m_editPos-1;
			else
				m_selStart = m_editPos;
			if ( m_selStart == m_selEnd+1 )
				ClearSelection();
		}
		InvalidateRect(NULL, FALSE);
	}
	else
	{
		if ( bControl && !bShift)
		{
			switch ( nChar )
			{
			case VK_HOME:
				m_editPos = 0;
				ChangeEditPos ( 0, 0, false);
				break;
			case VK_END:
				m_editPos = p_doc->size;
				ChangeEditPos ( 0, 0, false);
				break;
			case VK_INSERT:
				OnEditCopy();
				break;
			case 'A':
				OnEditSelectAll();
				break;
			case VK_TAB:	
				if ( m_digitsInData == 2 && m_bRealShowAscii )
				{	// switch hex/ascii
					m_editMode ^= EDIT_BYTES;
					ChangeEditPos(0,0, false);		// update scrollbars and caret pos
				}
				break;
			default:
				CView::OnKeyDown(nChar, nRepCnt, nFlags);
				break;
			}
		}
		else if ( !bControl && !bShift )
		{
			switch ( nChar )
			{
			case VK_LEFT:
				ChangeEditPos ( -1,  0, false);
				break;
			case VK_RIGHT:
				ChangeEditPos ( 1, 0, false);
				break;
			case VK_UP:
				ChangeEditPos ( 0, -1, false);
				break;
			case VK_DOWN:
				ChangeEditPos ( 0, 1, false);
				break;
			case VK_PRIOR:
				ChangeEditPos ( 0, -m_charCountWindow.cy, false);
				break;
			case VK_NEXT:
				ChangeEditPos ( 0, m_charCountWindow.cy, false);
				break;
			case VK_HOME:
				if ( m_editMode & EDIT_BYTES )
					m_nHorzScroll = 0;
				ChangeEditPos ( -(m_editPos % m_columns),  0, false);
				break;
			case VK_END:	// todo???: place caret at the end
				ChangeEditPos ( m_columns-(m_editPos % m_columns)-1,  0, false);
				break;
			case VK_BACK:
				if ( (m_editMode & EDIT_INSERT) || m_allowChangeSize )
				{
					if ( IsSelectionEmpty() )
					{
						if ( m_editPos > 0 )
						{
							ChangeEditPos ( -1,  0, true );
							DeleteData ( m_editPos );
							p_doc->SetModifiedFlag(TRUE);
						}
					}
					else
						DeleteSelection();
					InvalidateRect(NULL, FALSE);
					PlaceCaret();
				}
				break;
			case VK_DELETE:
				if ( (m_editMode & EDIT_INSERT) || m_allowChangeSize )
				{
					if ( IsSelectionEmpty() )
					{
						if ( (DWORD)m_editPos < p_doc->size )
						{
							DeleteData ( m_editPos );
							ChangeEditPos ( 0,  0, true );
							p_doc->SetModifiedFlag(TRUE);
						}
					}
					else
					{
						DeleteSelection();
						InvalidateRect(NULL, FALSE);
						PlaceCaret();
					}
				}
				else // if !EDIT_INSERT
				{
					if ( !IsSelectionEmpty() )
					{
						// zero selected bytes
						BYTE* buf;
						buf = p_doc->data;
						//m_data.AccessData((void**)&buf);
						memset ( buf+m_selStart*(m_digitsInData/2), 0, (m_selEnd-m_selStart+1)*(m_digitsInData/2));
						//m_data.UnaccessData();
						ClearSelection();
					}
				}
				break;
			case VK_INSERT:
				m_editMode ^= EDIT_INSERT;
				CreateSolidCaret ( (m_editMode & EDIT_INSERT) ? 0:m_cellSize.cx, m_cellSize.cy );
				ShowCaret();
				PlaceCaret();
				break;
			default:
				CView::OnKeyDown(nChar, nRepCnt, nFlags);
				return;
			}

			if ( !IsSelectionEmpty() )
			{
				ClearSelection();
				InvalidateRect(NULL, FALSE);
			}
		}
	}
	
	//CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CHexEditView::ChangeEditPos(long dx, long dy, BOOL bRepaint /* = false */)
{
	m_nEditDigit = 0;
	m_editPos += dy*m_columns + dx;
	if ( NormalizeEditPos())
		bRepaint = true;

	if ( UpdateScrollBars ())
		bRepaint = true;

	if ( bRepaint )
		InvalidateRect(NULL, FALSE);
	PlaceCaret();
}

void CHexEditView::ChangeEditPos(CPoint pt)
{
	pt.x = (pt.x - m_cellSize.cx/2) / m_cellSize.cx;
	pt.y /= m_cellSize.cy;

	pt.x += m_nHorzScroll;

	if ( pt.x >= m_asciiMargin && m_bRealShowAscii && !m_bMouseMove || 
		m_bMouseMove && !(m_editMode & EDIT_BYTES))	
		// click in the ascii field or extending selection in ascii field
		
		/* if selection was started in ascii, we never shouldn't switch 
		   to hex field until selection is finished
		   The same, when selection started in hex field
		 */
	{
		if ( pt.x < m_asciiMargin )
			pt.x = m_asciiMargin;
		else
		if ( pt.x >= m_asciiMargin + ((m_columns*m_digitsInData/2) * CHAR_PADDING) )
			pt.x = m_asciiMargin + ((m_columns*m_digitsInData/2) * CHAR_PADDING) - 1;
		m_editPos = m_viewPos + ((pt.x-m_asciiMargin)/CHAR_PADDING) + pt.y * m_columns;
		m_editMode &= ~EDIT_BYTES;
	}
	else
	{
		m_editMode |= EDIT_BYTES;
		if ( pt.x >= m_asciiMargin )
			pt.x = m_asciiMargin - 1;
		pt.x -= m_addrMargin;
		if ( pt.x < 0 )	// clicked in address area
			pt.x = 0;
		if ( pt.x >= m_columns*(m_digitsInData+1))
			pt.x = (m_columns-1)*(m_digitsInData+1);
		
		m_nEditDigit = pt.x % (m_digitsInData+1);
		if ( m_nEditDigit == m_digitsInData )
		{
			m_nEditDigit = m_digitsInData-1;
		}
		m_editPos = m_viewPos + pt.x / (m_digitsInData+1) + pt.y * m_columns;
	}

	NormalizeEditPos();
	UpdateScrollBars();
}

void CHexEditView::OnEditCut()
{
	if ( (m_editMode & EDIT_INSERT) || m_allowChangeSize )
	{
		if ( !IsSelectionEmpty() ) 
		{
			OnEditCopy();
			DeleteSelection();
			InvalidateRect(NULL, FALSE);
		}
	}
}

void CHexEditView::OnEditCopy()
{
	if ( IsSelectionEmpty() ) return;

	if ( !OpenClipboard() )
		return;
	VERIFY(::EmptyClipboard());

		// size of selection in bytes
	int	dwLen = (m_selEnd-m_selStart+1)*(m_digitsInData/2);

		// alloc memory for "BinaryData" clipboard format
	HGLOBAL	hMemb = ::GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_ZEROINIT, dwLen+sizeof(DWORD));
		// alloc memory for CF_TEXT clipboard format
	HGLOBAL	hMema = ::GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_ZEROINIT, dwLen+1);
	
	ASSERT ( hMemb );
	ASSERT ( hMema );
	
	// copy binary
	BYTE* buf;
	LPBYTE	p = (BYTE*)::GlobalLock(hMemb);

		// first DWORD in BinaryData contains the length of data in bytes
	*(DWORD*) p = dwLen;
	p += sizeof(DWORD);

	buf = p_doc->data;
	memcpy(p, buf+m_selStart*(m_digitsInData/2), dwLen);

	::GlobalUnlock(hMemb);
	
	// copy text
	p = (BYTE*)::GlobalLock(hMema);
	memcpy(p, buf+m_selStart*(m_digitsInData/2), dwLen);
	p[dwLen] = 0;

	::GlobalUnlock(hMema);

	VERIFY(::SetClipboardData((USHORT)RegisterClipboardFormat(_T("BinaryData")), hMemb));
	VERIFY(::SetClipboardData(CF_TEXT, hMema));
	::CloseClipboard();
}

void CHexEditView::OnEditPaste()
{
	BOOL bIsBinaryData = FALSE;
	HGLOBAL hmem = NULL;
	UINT hFormat = RegisterClipboardFormat(_T("BinaryData"));
	// we can paste CF_TEXT or BinaryData
	if (!::IsClipboardFormatAvailable((USHORT)hFormat) && !::IsClipboardFormatAvailable(CF_TEXT))
		return;
	
	VERIFY ( OpenClipboard() );
	if (::IsClipboardFormatAvailable((USHORT)hFormat))
	{
		hmem = ::GetClipboardData((USHORT)hFormat);
		bIsBinaryData = TRUE;
	}
	else if (::IsClipboardFormatAvailable(CF_TEXT))
	{
		hmem = ::GetClipboardData(CF_TEXT);
	}
	
	if(hmem)
	{
		LPBYTE	p = (BYTE*)::GlobalLock(hmem);
		DWORD	dwLen;
		if ( bIsBinaryData )
		{
			// first DWORD in BinaryData contains the length of data in bytes
			dwLen = *(DWORD*)p;
			p += sizeof(DWORD);
		}
		else
			dwLen = (DWORD) ::GlobalSize(hmem)-1;
		
		if ( dwLen > 0 )
		{
			if ( m_editMode & EDIT_INSERT )
			{
				DeleteSelection();
				InsertData ( m_editPos, (dwLen+m_digitsInData/2-1)/(m_digitsInData/2) );
			}
			else 
			{
				if ( !IsSelectionEmpty() )
				{
					m_editPos = m_selStart;
					if ( dwLen > ULONG(m_selEnd - m_selStart + 1) )
						dwLen = m_selEnd - m_selStart + 1;
					ClearSelection();
				}
				
				DWORD dwDataLen = p_doc->size;
				if ( m_editPos*(m_digitsInData/2)+dwLen > dwDataLen )
				{
					// need to add bytes at the end
					if (m_allowChangeSize)
						InsertData ( dwDataLen/(m_digitsInData/2), (dwLen - (dwDataLen-m_editPos*(m_digitsInData/2))+(m_digitsInData/2)-1)/(m_digitsInData/2) );
					else
						dwLen = dwDataLen - m_editPos*(m_digitsInData/2);
				}
			}
			
			if ( dwLen > 0 )
			{
				BYTE* buf;
				buf = p_doc->data;
				memcpy ( buf+m_editPos*(m_digitsInData/2), p, dwLen );
				p_doc->SetModifiedFlag(TRUE);
				InvalidateRect(NULL, FALSE);
			}
		}
		::GlobalUnlock(hmem);
	}
	::CloseClipboard();
}

void CHexEditView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!IsSelectionEmpty());
}

void CHexEditView::OnUpdateEditCut(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!IsSelectionEmpty() && ((m_editMode & EDIT_INSERT) || m_allowChangeSize));
}

void CHexEditView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	//COleDataObject obj;
	//if (obj.AttachClipboard())
	//	pCmdUI->Enable(obj.IsDataAvailable(CF_TEXT) || obj.IsDataAvailable((USHORT)RegisterClipboardFormat(_T("BinaryData"))));
	//else
		pCmdUI->Enable(false);
}

void CHexEditView::OnEditSelectAll()
{
	if ( p_doc->size == 0 ) 
		return;
	m_editPos = m_selStart = 0;
	m_selEnd = p_doc->size/(m_digitsInData/2)-1;
	ChangeEditPos (0, 0, true );	// place caret & repaint
}

void CHexEditView::OnUpdateEditSelectAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(p_doc->size != 0);
}

void CHexEditView::DeleteData(DWORD dwPos, DWORD dwCount /*=1*/)
{
	// convert to bytes
	dwPos *= (m_digitsInData/2);
	dwCount *= (m_digitsInData/2);

	DWORD dwArrSize = p_doc->size;
	ASSERT ( dwCount > 0 );
	ASSERT ( dwPos < dwArrSize );

	if ( dwPos != dwArrSize-1 )
	{
		BYTE* buf;
		buf = p_doc->data;
		memmove ( buf+dwPos, buf+dwPos+dwCount, dwArrSize-dwPos-dwCount );
	}

	p_doc->size = dwArrSize - dwCount;
	RecalcLayout();
}

void CHexEditView::DeleteSelection()
{
	if ( !IsSelectionEmpty())
	{
		DeleteData ( m_selStart, m_selEnd-m_selStart+1 );
		m_editPos = m_selStart;
		p_doc->SetModifiedFlag(TRUE);
		ClearSelection();
		NormalizeEditPos();
		PlaceCaret();
	}
}

void CHexEditView::InsertData(DWORD dwPos, DWORD dwCount/*=1*/)
{
	// convert to bytes
	dwPos *= (m_digitsInData/2);
	dwCount *= (m_digitsInData/2);

	DWORD dwArrSize = p_doc->size;
	ASSERT ( dwCount > 0 );
	ASSERT ( dwPos <= dwArrSize );

	if ( dwArrSize+dwCount <= dwArrSize || dwArrSize+dwCount >= 0x80000000 )
	{
		// overflow
		AfxThrowMemoryException ();
		return;
	}

	p_doc->size = dwArrSize + dwCount;

	if ( dwPos != dwArrSize )
	{
		BYTE* buf;
		buf = p_doc->data;

		memmove ( buf+dwPos+dwCount, buf+dwPos, dwArrSize-dwPos );
		memset ( buf+dwPos, 0, dwCount );
	}
	RecalcLayout();
}

void CHexEditView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if(!p_doc->data)
		return;

	if ( m_editMode & EDIT_BYTES )
	{
		if((nChar >= '0' && nChar <= '9') || (nChar >= 'a' && nChar <= 'f') || (nChar >= 'A' && nChar <= 'F'))
		{
			UINT b = nChar - '0';
			if(b > 9)
			{
				if (b <= 'F' - '0')
					b = 10 + nChar - 'A';
				else
					b = 10 + nChar - 'a';
			}
			UINT mask = 0xFUL << (( m_digitsInData - m_nEditDigit - 1 )*4);

			if ( !EnterNumber())
				return;

			BYTE* buf;
			DWORD bufAddr = m_editPos*(m_digitsInData/2);
			buf = p_doc->data;
			*(DWORD*)(buf+bufAddr) = *(DWORD*)(buf+bufAddr) & ~mask | (b << (( m_digitsInData - m_nEditDigit - 1 )*4));
			
			m_nEditDigit++;
			if (NormalizeEditPos())		
				InvalidateRect(NULL, FALSE);			// cursor was outside visible area
			InvalidateEditRect ( m_editPos );
			if ( m_nEditDigit == m_digitsInData )
				ChangeEditPos(1, 0, false);
			else
			{
				UpdateScrollBars();
				PlaceCaret();
			}
		}	
		else
			CView::OnChar(nChar, nRepCnt, nFlags);
	}
	else	// EDIT_ASCII
	{
		if(nChar >= ' ')
		{
			if (!EnterNumber())
				return;

			CString s;
			DWORD i;

			s = (_TCHAR) nChar;

			if(MakeHex(s, i, 1) == FALSE)
				return;

			BYTE* buf;
			buf = p_doc->data;
			buf[m_editPos] = (BYTE) i;
			p_doc->SetModifiedFlag(TRUE);
			if (NormalizeEditPos())		
				InvalidateRect(NULL, FALSE);			// cursor was outside visible area
			else
				InvalidateEditRect ( m_editPos );
			ChangeEditPos(1, 0, false);
		}
	}
}

BOOL CHexEditView::EnterNumber()
{
	if ( !IsSelectionEmpty())
	{
		if ( (m_editMode & EDIT_INSERT) || m_allowChangeSize )
		{
			// delete current selection and insert 1 number
			m_editPos = m_selStart;
			DeleteSelection();
			InsertData ( m_editPos, 1 );
			InvalidateRect(NULL, FALSE);
		}
		else
		{
			ClearSelection();
		}
	}
	else	// is m_editPos at the end?
		if ( DWORD(m_editPos)*(m_digitsInData/2) == p_doc->size)
		{
			if ( (m_editMode & EDIT_INSERT) || m_allowChangeSize )
			{
				InsertData ( m_editPos, 1 );
				if ( m_editPos % m_columns != m_columns-1)
					InvalidateEditRect (m_editPos);
				else
					InvalidateRect(NULL, FALSE);	// we need to add new line
			}
			else
				return false;
		}
		else if	((m_editMode & EDIT_INSERT) && m_nEditDigit == 0 )
			// insert 1 number
		{
			InsertData ( m_editPos, 1 );
			InvalidateRect(NULL, FALSE);
		}
	p_doc->SetModifiedFlag(TRUE);
	return true;
}

void CHexEditView::InvalidateEditRect(long nEditPos)
{
	CRect rcInvalid;
	long nPos = nEditPos - m_viewPos;
	ASSERT ( nPos >= 0 && nPos < m_columns*m_charCountWindow.cy );

	// invalidate rect of the number
	rcInvalid.left = (nPos % m_columns * (m_digitsInData+1) + m_addrMargin - m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2;
	rcInvalid.top = nPos / m_columns * m_cellSize.cy;
	rcInvalid.right = rcInvalid.left + (m_digitsInData+1)*m_cellSize.cx;
	rcInvalid.bottom = rcInvalid.top + m_cellSize.cy;
	InvalidateRect(rcInvalid);

	if ( m_bRealShowAscii )
	{
		// invalidate rect of corresponding ASCII char
		rcInvalid.left = (((nPos % m_columns) * CHAR_PADDING)+ m_asciiMargin - m_nHorzScroll)*m_cellSize.cx+m_cellSize.cx/2;
		rcInvalid.top = nPos / m_columns * m_cellSize.cy;
		rcInvalid.right = rcInvalid.left + (m_cellSize.cx * CHAR_PADDING);
		rcInvalid.bottom = rcInvalid.top + m_cellSize.cy;
		InvalidateRect(rcInvalid);
	}
}

void CHexEditView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	m_bMouseDown = true;

	if ( !IsSelectionEmpty() )
		ClearSelection();

	if ( nFlags & MK_SHIFT )
	{
		if ( DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
			m_posMouseDown = m_editPos - 1;
		else
			m_posMouseDown = m_editPos;
		RecalcSelection(point);
		m_bMouseMove = true;
	}
	else
	{
		ChangeEditPos ( point );
		if ( DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
			m_posMouseDown = m_editPos - 1;
		else
			m_posMouseDown = m_editPos;
	}
	SetCapture();
	HideCaret();
	CView::OnLButtonDown(nFlags, point);
}

void CHexEditView::RecalcSelection(CPoint point)
{
	if ( p_doc->size == 0 )
		return;
	if ( m_bMouseDown )
	{
		int nPrevSelStart = m_selStart, nPrevSelEnd = m_selEnd;

		ChangeEditPos ( point );
		if ( m_posMouseDown < m_editPos )
		{
			if ( DWORD(m_editPos*(m_digitsInData/2)) == p_doc->size)
				m_editPos--;
			m_selStart = m_posMouseDown; 
			m_selEnd = m_editPos;
			m_editPos = m_selEnd;
			m_nEditDigit = 0;
		}
		else
		{
			m_selStart = m_editPos;
			m_selEnd = m_posMouseDown; 
			m_editPos = m_selStart;
			m_nEditDigit = 0;
		}
		if ( nPrevSelStart != m_selStart || nPrevSelEnd != m_selEnd )
			InvalidateRect(NULL, FALSE);
	}
}

void CHexEditView::OnLButtonUp(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	if ( m_bMouseDown )
	{
		if ( m_bMouseMove )
			RecalcSelection(point);
		ReleaseCapture();
		m_bMouseDown = false;
		m_bMouseMove = false;
		if ( m_bTimer )
		{
			KillTimer(ID_TIMER);
			m_bTimer = false;
		}

		// place caret just after the selection
		if ( !IsSelectionEmpty() )
		{
			if ( m_editPos == m_selEnd )
				m_editPos++;
			m_nEditDigit = 0;
		}
	}
	PlaceCaret();
	ShowCaret();
	CView::OnLButtonUp(nFlags, point);
}

void CHexEditView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	ChangeEditPos ( point );
	
	// select the number under cursor if it isn't at the end of data
	if ( DWORD(m_editPos*(m_digitsInData/2)) != p_doc->size)
	{
		m_selStart = m_selEnd = m_editPos;
		m_nEditDigit = 0;
		InvalidateEditRect(m_editPos);
		m_editPos++;
	}

	PlaceCaret();
	m_bMouseDown = false;
	ReleaseCapture();

	CView::OnLButtonDblClk(nFlags, point);
}

UINT CHexEditView::OnGetDlgCode()
{
	return CView::OnGetDlgCode() | DLGC_WANTARROWS | DLGC_WANTCHARS | DLGC_WANTTAB;
}

void CHexEditView::OnMouseMove(UINT nFlags, CPoint point)
{
	if(!p_doc->data)
		return;

	if ( m_bMouseDown )
	{
		m_bMouseMove = true;
		if ( !m_bTimer && !m_rcClient.PtInRect(point))
		{
			// mouse went outside the window, so we need to scroll even if mouse doesn't 
			// moving
			SetTimer ( ID_TIMER, 200, 0 );
			m_bTimer = true;
		}
		else if ( m_bTimer && m_rcClient.PtInRect(point))
		{
			// mouse is inside window, so we don't need the timer
			KillTimer ( ID_TIMER );
			m_bTimer = false;
		}
		m_prevMousePoint = point;
		RecalcSelection(point);
	}
	CView::OnMouseMove(nFlags, point);
}

void CHexEditView::OnHScroll(UINT nSBCode, UINT /*nPos*/, CScrollBar* /*pScrollBar*/) 
{
	int nPrevHorzScroll = m_nHorzScroll;
	switch(nSBCode)
	{
	case SB_LINELEFT:
		m_nHorzScroll--;
		break;
	case SB_PAGELEFT:
		m_nHorzScroll -= m_charCountWindow.cx;
		break;
	case SB_LINERIGHT:
		m_nHorzScroll++;
		break;
	case SB_PAGERIGHT:
		m_nHorzScroll += m_charCountWindow.cx;
		break;
	case SB_THUMBTRACK:
		SCROLLINFO si;

		si.cbSize = sizeof(si);
		GetScrollInfo ( SB_HORZ, &si, SIF_TRACKPOS );
		m_nHorzScroll = si.nTrackPos;
		break;
	}
	if ( m_nHorzScroll < 0 )
		m_nHorzScroll = 0;
	else if ( m_nHorzScroll > (m_charCountView.cx - m_charCountWindow.cx) )
		m_nHorzScroll = (m_charCountView.cx - m_charCountWindow.cx);

	if ( nPrevHorzScroll != m_nHorzScroll )
	{
		InvalidateRect(NULL, FALSE);
		SetScrollPos ( SB_HORZ, m_nHorzScroll );
		PlaceCaret();
	}
}

void CHexEditView::OnTimer(UINT nIDEvent)
{
	// called when mouse went outside window during selection
	ASSERT ( m_bTimer );

	if ( nIDEvent == ID_TIMER )
	{
		RecalcSelection ( m_prevMousePoint );
	}
	else
		CView::OnTimer(nIDEvent);
}

void CHexEditView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	if(!p_doc->data)
		return;

	RecalcLayout();
	InvalidateRect(NULL, FALSE);
}

BOOL CHexEditView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	m_viewPos += (-zDelta / 40) * m_columns;

	if ( m_viewPos < 0)
	{
		m_viewPos = 0;
	}
	else if ( DWORD(m_viewPos) > p_doc->size / (m_digitsInData/2) - m_columns*m_charCountWindow.cy)
	{
		m_viewPos = p_doc->size / (m_digitsInData/2) / m_columns * m_columns - m_columns*m_charCountWindow.cy;
		if ( m_viewPos < 0 ) m_viewPos = 0;
	}

	UpdateScrollBars();
	InvalidateRect(NULL, FALSE);
	PlaceCaret();

	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CHexEditView::SetColour(COLORREF colour, DWORD colour_def)
{
	m_colour_data[colour_def] = colour;
}

BOOL CHexEditView::MakeString(DWORD i, CString& s, BYTE bytes)
{
	if(bytes == 1)
	{
		if((_istprint((_TCHAR) i) != 0) && (i >= 0x20))
		{
			s = (_TCHAR) i;
			return TRUE;
		}
	}
	return FALSE;
}

BOOL CHexEditView::MakeHex(CString s, DWORD &i, BYTE bytes)
{
	if(bytes == 1)
	{
		i = s[0];
		return TRUE;
	}
	return FALSE;
}

BOOL CHexEditView::OnEraseBkgnd(CDC* pDC) 
{	
	CBrush backBrush(m_colour_data[BACK_COLOUR]);
	CRect rect;
	pDC->GetClipBox(&rect);
	pDC->FillRect(rect, &backBrush);
	return TRUE;
}

COLORREF hex_edit_def_colours[CHexEditView::COLOUR_COUNT] =
{
	RGB(0xFF, 0xFF, 0xFF),
	RGB(0x80, 0x80, 0x80),
	RGB(0x00, 0x00, 0x00),
	RGB(0x00, 0x00, 0x00),
	RGB(0x00, 0x00, 0xFF),
	RGB(0xFF, 0x00, 0x00),
	RGB(0x80, 0x00, 0x80),
	RGB(0x80, 0x00, 0x00),
	RGB(0x00, 0xC0, 0x00),
	RGB(0x00, 0x80, 0x00),
};

void CBaseView::StorePosition(void)
{
}

void CBaseView::RestorePosition(void)
{
}

void CBaseView::ChangeView()
{
}

void CBaseView::JumpToSelected(DWORD start, DWORD end)
{
}


void CHexEditView::OnContextMenu(CWnd* /*pWnd*/, CPoint point)
{
	// make sure window is active
	GetParentFrame()->ActivateFrame();
	UpdateWindow();

	CMenu menu;
	menu.LoadMenu(IDR_CONTEXT_HEX_EDIT);

	CMenu* pPopup = menu.GetSubMenu(0);
	ASSERT(pPopup != NULL);

	// route commands through main window
	pPopup->TrackPopupMenu(TPM_RIGHTBUTTON | TPM_LEFTALIGN, point.x, point.y, AfxGetMainWnd());
}
