// SearchR.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "SearchR.h"
#include "SearchRDoc.h"
#include "HexEditView.h"
#include "HexView.h"
#include "SearchDialog.h"
#include "SearchResults.h"
#include "SearchRView.h"
#include "TableInfo.h"
#include "TableView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSearchRApp

BEGIN_MESSAGE_MAP(CSearchRApp, CWinApp)
	//{{AFX_MSG_MAP(CSearchRApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CSearchRApp construction

CSearchRApp::CSearchRApp()
{
#ifdef WINDOWS_XP_BUILD
	EnableHtmlHelp();
#endif
}


// The one and only CSearchRApp object

CSearchRApp theApp;

// CSearchRApp initialization

BOOL CSearchRApp::InitInstance()
{
	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}
	AfxEnableControlContainer();
	SetRegistryKey(_T("DeJap Translations"));
	// Load standard INI file options (including MRU)
	LoadStdProfileSettings(4);

	SearchRConfig::LoadConfig();

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views
	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CSearchRDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CHexView));
	AddDocTemplate(pDocTemplate);
	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
	// Dispatch commands specified on the command line.  Will return FALSE if
	// app was launched with /RegServer, /Register, /Unregserver or /Unregister.
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
	// The one and only window has been initialized, so show and update it
	m_pMainWnd->ShowWindow(SW_SHOWMAXIMIZED);
	m_pMainWnd->UpdateWindow();
	// call DragAcceptFiles only if there's a suffix
	//  In an SDI app, this should occur after ProcessShellCommand
	return TRUE;
}



// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum IDD_DEF { IDD = IDD_ABOUTBOX };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

// App command to run the dialog
void CSearchRApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}


// CSearchRApp message handlers

int CSearchRApp::ExitInstance()
{
	SearchRConfig::SaveConfig();
	return CWinApp::ExitInstance();
}

namespace SearchRConfig
{
	CString search_string[SEARCH_HISTORY];
	CString table_mru_list[TABLE_MRU_SIZE];
	BOOL deinterleave_smd_roms;
	COLORREF hex_view_colour_data[3][CHexEditView::COLOUR_COUNT];

	DWORD store_m_dwStartAddr;
	DWORD store_m_nEditDigit;
	DWORD store_m_selStart;
	DWORD store_m_selEnd;
	DWORD store_m_editPos;
	DWORD store_m_viewPos;
	DWORD store_m_nHorzScroll;
	DWORD store_m_editMode;

	CRect winpos_bookmarks;
	CRect winpos_search_results;
	CRect winpos_table_info;
	CRect winpos_gfx_mode_info;
	CRect winpos_gfx_tile_edit;

	void LoadRect(CRect *rect, LPCTSTR name)
	{
		CString id;

		id.Format(_T("%s Top"), name);
		rect->top = theApp.GetProfileInt(_T("Options"), id, 0);
		id.Format(_T("%s Bottom"), name);
		rect->bottom = theApp.GetProfileInt(_T("Options"), id, 0);
		id.Format(_T("%s Left"), name);
		rect->left = theApp.GetProfileInt(_T("Options"), id, 0);
		id.Format(_T("%s Right"), name);
		rect->right = theApp.GetProfileInt(_T("Options"), id, 0);
	}

	void SaveRect(CRect *rect, LPCTSTR name)
	{
		CString id;

		id.Format(_T("%s Top"), name);
		theApp.WriteProfileInt(_T("Options"), id, rect->top);
		id.Format(_T("%s Bottom"), name);
		theApp.WriteProfileInt(_T("Options"), id, rect->bottom);
		id.Format(_T("%s Left"), name);
		theApp.WriteProfileInt(_T("Options"), id, rect->left);
		id.Format(_T("%s Right"), name);
		theApp.WriteProfileInt(_T("Options"), id, rect->right);
	}

	void LoadConfig(void)
	{
		DWORD i;
		CString id;

		for(i = 0; i < SEARCH_HISTORY; i ++)
		{
			id.Format(_T("Search String %d"), i + 1);
			search_string[i] = theApp.GetProfileString(_T("Options"), id, _T(""));
		}
		for(i = 0; i < TABLE_MRU_SIZE; i ++)
		{
			id.Format(_T("Table List %d"), i + 1);
			table_mru_list[i] = theApp.GetProfileString(_T("Options"), id, _T(""));
		}
		deinterleave_smd_roms = theApp.GetProfileInt(_T("Options"), _T("DeInterleave .smd ROMS"), TRUE);
		for(i = 0; i < CHexEditView::COLOUR_COUNT; i ++)
		{
			id.Format(_T("Hex View Colour %d"), i + 1);
			hex_view_colour_data[0][i] = theApp.GetProfileInt(_T("Options"), id, hex_edit_def_colours[i]);
		}
		i = 0;
		id.Format(_T("Relative View Colour %d"), i + 1);
		hex_view_colour_data[1][i] = theApp.GetProfileInt(_T("Options"), id, RGB(0xFF, 0xFF, 0xF0));
		id.Format(_T("Table View Colour %d"), i + 1);
		hex_view_colour_data[2][i] = theApp.GetProfileInt(_T("Options"), id, RGB(0xF8, 0xF8, 0xFF));

		LoadRect(&winpos_bookmarks, _T("Window Bookmarks"));
		LoadRect(&winpos_search_results, _T("Window Search Results"));
		LoadRect(&winpos_table_info, _T("Window Table Information"));
		LoadRect(&winpos_gfx_mode_info, _T("Window Gfx Mode Information"));
		LoadRect(&winpos_gfx_tile_edit, _T("Window Gfx Tile Editor"));
	}

	void SaveConfig(void)
	{
		DWORD i;
		CString id;

		for(i = 0; i < SEARCH_HISTORY; i ++)
		{
			id.Format(_T("Search String %d"), i + 1);
			theApp.WriteProfileString(_T("Options"), id, search_string[i]);
		}
		for(i = 0; i < TABLE_MRU_SIZE; i ++)
		{
			id.Format(_T("Table List %d"), i + 1);
			theApp.WriteProfileString(_T("Options"), id, table_mru_list[i]);
		}
		theApp.WriteProfileInt(_T("Options"), _T("DeInterleave .smd ROMS"), deinterleave_smd_roms);
		for(i = 0; i < CHexEditView::COLOUR_COUNT; i ++)
		{
			id.Format(_T("Hex View Colour %d"), i + 1);
			theApp.WriteProfileInt(_T("Options"), id, hex_view_colour_data[0][i]);
		}
		i = 0;
		id.Format(_T("Relative View Colour %d"), i + 1);
		theApp.WriteProfileInt(_T("Options"), id, hex_view_colour_data[1][i]);
		id.Format(_T("Table View Colour %d"), i + 1);
		theApp.WriteProfileInt(_T("Options"), id, hex_view_colour_data[2][i]);

		SaveRect(&winpos_bookmarks, _T("Window Bookmarks"));
		SaveRect(&winpos_search_results, _T("Window Search Results"));
		SaveRect(&winpos_table_info, _T("Window Table Information"));
		SaveRect(&winpos_gfx_mode_info, _T("Window Gfx Mode Information"));
		SaveRect(&winpos_gfx_tile_edit, _T("Window Gfx Tile Editor"));
	}
};
