// SearchRDoc.cpp : implementation of the CSearchRDoc class
//

#include "stdafx.h"
#include "SearchR.h"
#include "TableDoc.h"
#include "SearchRDoc.h"
#include "HexEditView.h"
#include "HexView.h"
#include "SearchDialog.h"
#include "SearchResults.h"
#include "SearchRView.h"
#include "TableInfo.h"
#include "TableView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CSearchRDoc *p_doc;

CSearchResult::CSearchResult()
{
	offset = 0;
	length = 0;
	next_char = 1;
	next = NULL;
	byte_a = 'a';
	byte_A = 'A';
	byte_0 = '0';
	a_found = FALSE;
	A_found = FALSE;
	found_0 = FALSE;

	DWORD i;

	for(i = 0; i < 256; i ++)
	{
		lookup[i] = (BYTE) i;
		lookup_valid[i] = FALSE;
	}
}

// CSearchRDoc

IMPLEMENT_DYNCREATE(CSearchRDoc, CDocument)

BEGIN_MESSAGE_MAP(CSearchRDoc, CDocument)
	//{{AFX_MSG_MAP(CSearchRDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CSearchRDoc construction/destruction

CSearchRDoc::CSearchRDoc()
: search_id(false)
{
	p_doc = this;

	size = 0;
	data = NULL;
	result_count = 0;
	result_data = NULL;
	search_id = 0;

	p_table_doc = new CTableDoc;
}

CSearchRDoc::~CSearchRDoc()
{
	DeleteResults();

	if(data)
		free(data);
}

BOOL CSearchRDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}




// CSearchRDoc serialization

void CSearchRDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar.Write(data, size);
	}
	else
	{
		CFile* file_in;
		file_in = ar.GetFile();

		size = (DWORD) file_in->GetLength();

		// genesis roms (.smd format)

		if(((size & 0x3FFF) == 0x200) && (_tcsstr(file_in->GetFileName(), _T(".smd"))) && (SearchRConfig::deinterleave_smd_roms))
		{
			BYTE *temp;
			DWORD i, j;

			size -= 0x200;

			data = (BYTE *) malloc(size);
			temp = (BYTE *) malloc(0x4000);
			ar.Read(temp, 0x200);
			
			i = 0;
			while(i < size)
			{
				ar.Read(temp, 0x4000);
				j = 0;
				while(j < 0x2000)
				{
					data[i + (j * 2) + 1] = temp[j + 0x0000];
					data[i + (j * 2) + 0] = temp[j + 0x2000];
					j ++;
				}
				i += 0x4000;
			}

			free(temp);
			p_mainframe->UpdateCurrentView();
			return;
		}

		// snes roms (.smc/.sfc format)

		if(((size & 0x7FFF) == 0x200) && ((_tcsstr(file_in->GetFileName(), _T(".smc"))) || (_tcsstr(file_in->GetFileName(), _T(".sfc")))) && (SearchRConfig::deinterleave_smd_roms))
		{
			size -= 0x200;

			data = (BYTE *) malloc(size);

			ar.Read(data, 0x200);
			ar.Read(data, size);

			p_mainframe->UpdateCurrentView();
			return;
		}

		data = (BYTE *) malloc(size);
		ar.Read(data, size);
		p_mainframe->UpdateCurrentView();
		return;

	}
}

// CSearchRDoc diagnostics

#ifdef _DEBUG
void CSearchRDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSearchRDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CSearchRDoc commands

void CSearchRDoc::DeleteResults(void)
{
	CSearchResult *next;

	while(result_data)
	{
		next = result_data->next;
		delete result_data;
		result_data = next;
	}
	result_count = 0;
}

void CSearchRDoc::DoSearchRelative(LPCTSTR s, DWORD next_char, DWORD sequence_check)
{
	DWORD i, j, k;
	BOOL match;
	BYTE byte_a;
	BYTE byte_A;
	BYTE byte_0;
	BOOL a_found;
	BOOL A_found;
	BOOL found_0;

	DWORD search_pos[3][16];
	_TCHAR search_data[3][16];
	DWORD search_count[3];

	search_id ++;

	// build search data...

	search_count[0] = 0;
	search_count[1] = 0;
	search_count[2] = 0;
	i = 0;
	while(s[i])
	{
		if((s[i] >= 'a') && (s[i] <= 'z'))
		{
			if(search_count[0] < 16)
			{
				search_data[0][search_count[0]] = s[i];
				search_pos[0][search_count[0]] = i;
				search_count[0] ++;
			}
		}
		if((s[i] >= 'A') && (s[i] <= 'Z'))
		{
			if(search_count[1] < 16)
			{
				search_data[1][search_count[1]] = s[i];
				search_pos[1][search_count[1]] = i;
				search_count[1] ++;
			}
		}
		if((s[i] >= '0') && (s[i] <= '9'))
		{
			if(search_count[2] < 16)
			{
				search_data[2][search_count[2]] = s[i];
				search_pos[2][search_count[2]] = i;
				search_count[2] ++;
			}
		}
		i ++;
	}

	CSearchResult *curr_result, *next_result;

	DeleteResults();
	curr_result = NULL;

	// skip insufficient search data...

	if((search_count[0] < 2) && (search_count[1] < 2) && (search_count[2] < 2))
		return;

	// search...

	byte_a = 'a';
	byte_A = 'A';
	byte_0 = '0';
	for(i = 0; i <= (size - _tcsclen(s)); i ++)
	{
		match = TRUE;
		a_found = FALSE;
		A_found = FALSE;
		found_0 = FALSE;
		for(k = 0; (k < 3) && match; k ++)
		{
			if(search_count[k] > 0)
			{
				if(search_count[k] < 2)
				{
					if(k == 0)
					{
						if(a_found == FALSE)
						{
							byte_a = (BYTE) (data[i + search_pos[k][0]] - ((search_data[k][0] - 'a') * next_char));
							a_found = TRUE;
						}
					}
					if(k == 1)
					{
						if(A_found == FALSE)
						{
							byte_A = (BYTE) (data[i + search_pos[k][0]] - ((search_data[k][0] - 'A') * next_char));
							A_found = TRUE;
						}
					}
					if(k == 2)
					{
						if(found_0 == FALSE)
						{
							byte_0 = (BYTE) (data[i + search_pos[k][0]] - ((search_data[k][0] - '0') * next_char));
							found_0 = TRUE;
						}
					}
				}
				else
				{
					for(j = 0; (j < (search_count[k] - 1)) && match; j ++)
					{
						if(((BYTE) (data[i + search_pos[k][j]] - data[i + search_pos[k][j + 1]])) == ((BYTE) ((search_data[k][j] - search_data[k][j + 1]) * next_char)))
						{
							if(k == 0)
							{
								if(a_found == FALSE)
								{
									byte_a = (BYTE) (data[i + search_pos[k][j]] - ((search_data[k][j] - 'a') * next_char));
									a_found = TRUE;
								}
							}
							if(k == 1)
							{
								if(A_found == FALSE)
								{
									byte_A = (BYTE) (data[i + search_pos[k][j]] - ((search_data[k][j] - 'A') * next_char));
									A_found = TRUE;
								}
							}
							if(k == 2)
							{
								if(found_0 == FALSE)
								{
									byte_0 = (BYTE) (data[i + search_pos[k][j]] - ((search_data[k][j] - '0') * next_char));
									found_0 = TRUE;
								}
							}
						}
						else
							match = FALSE;
					}
				}
			}
		}

		if(match == TRUE)
		{
			next_result = new CSearchResult;
			next_result->offset = i;
			next_result->next_char = next_char;
			next_result->length = (DWORD) _tcsclen(s);
			next_result->byte_a = byte_a;
			next_result->byte_A = byte_A;
			next_result->byte_0 = byte_0;
			next_result->a_found = a_found;
			next_result->A_found = A_found;
			next_result->found_0 = found_0;

			DWORD x, y;

			if(next_result->a_found == TRUE)
			{
				for(x = 0; x < 26; x ++)
				{
					y = (next_result->byte_a + (x * next_result->next_char)) & 0xFF;
					if(next_result->lookup_valid[y] == FALSE)
					{
						next_result->lookup_valid[y] = TRUE;
						next_result->lookup[y] = (BYTE) (x + 'a');
					}
					else
						match = FALSE;
				}
			}

			if(next_result->A_found == TRUE)
			{
				for(x = 0; x < 26; x ++)
				{
					y = (next_result->byte_A + (x * next_result->next_char)) & 0xFF;
					if(next_result->lookup_valid[y] == FALSE)
					{
						next_result->lookup_valid[y] = TRUE;
						next_result->lookup[y] = (BYTE) (x + 'A');
					}
					else
						match = FALSE;
				}
			}

			if(next_result->found_0 == TRUE)
			{
				for(x = 0; x < 10; x ++)
				{
					y = (next_result->byte_0 + (x * next_result->next_char)) & 0xFF;
					if(next_result->lookup_valid[y] == FALSE)
					{
						next_result->lookup_valid[y] = TRUE;
						next_result->lookup[y] = (BYTE) (x + '0');
					}
					else
						match = FALSE;
				}
			}

			if(match == TRUE)
			{
				if(!curr_result)
					result_data = next_result;
				else
					curr_result->next = next_result;

				curr_result = next_result;

				result_count ++;
				if(result_count >= 1000)
					return;
			}
			else
				delete next_result;
		}
	}
}

void CSearchRDoc::OnCloseDocument()
{
	CDocument::OnCloseDocument();
}

void CSearchRDoc::SetModifiedFlag(BOOL bModified)
{
	if(IsModified() == FALSE)
	{
		CDocument::SetModifiedFlag(bModified);
		p_mainframe->OnUpdateFrameTitle(TRUE);
	}
	else
		CDocument::SetModifiedFlag(bModified);
}