// SearchResults.cpp : implementation file
//

#include "stdafx.h"
#include "SearchR.h"
#include "SearchResults.h"
#include "SearchDialog.h"
#include "SearchRDoc.h"
#include "HexEditView.h"
#include "SearchRView.h"
#include "SearchAddToTable.h"


// CSearchResults dialog

IMPLEMENT_DYNAMIC(CSearchResults, CDialog)

CSearchResults::CSearchResults(CWnd* pParent /*=NULL*/)
	: CDialog(CSearchResults::IDD, pParent)
	, search_id(0)
{
	ASSERT(pParent != NULL);

	m_pParent = pParent;
	m_nID = CSearchResults::IDD;
	search_id = 0;
}

CSearchResults::~CSearchResults()
{
}

void CSearchResults::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SEARCH_RESULT_LIST, m_result_list);
	DDX_Control(pDX, IDC_SR_VIEW, m_view_result);
	DDX_Control(pDX, ID_EDIT_SEARCH, m_new_search);
	DDX_Control(pDX, IDC_SR_ADD_TO_TABLE, m_add_to_table);
}


BEGIN_MESSAGE_MAP(CSearchResults, CDialog)
	//{{AFX_MSG_MAP(CSearchResults)
	ON_COMMAND(ID_EDIT_SEARCH, OnEditSearch)
	ON_BN_CLICKED(IDC_SR_VIEW, OnBnClickedSrView)
	ON_NOTIFY(NM_CLICK, IDC_SEARCH_RESULT_LIST, OnNMClickSearchResultList)
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_SR_ADD_TO_TABLE, OnBnClickedSrAddToTable)
	ON_NOTIFY(NM_DBLCLK, IDC_SEARCH_RESULT_LIST, OnDblclkSearchResultList)
	//}}AFX_MSG_MAP
	ON_WM_WINDOWPOSCHANGED()
END_MESSAGE_MAP()


// CSearchResults message handlers

BOOL CSearchResults::Create()
{
	return CDialog::Create(m_nID, m_pParent);
}


BOOL CSearchResults::OnInitDialog()
{
	CDialog::OnInitDialog();

	LOGFONT lf;
	
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = 10;
	_tcscpy(lf.lfFaceName, _T("Courier"));
	m_font.CreateFontIndirect(&lf);
	m_result_list.SetFont(&m_font);

	m_result_list.InsertColumn(0, _T("Address"), LVCFMT_LEFT, 96, -1);
	m_result_list.InsertColumn(1, _T("a"), LVCFMT_LEFT, 32, -1);
	m_result_list.InsertColumn(2, _T("A"), LVCFMT_LEFT, 32, -1);
	m_result_list.InsertColumn(3, _T("1"), LVCFMT_LEFT, 32, -1);
	m_result_list.InsertColumn(4, _T("Relative"), LVCFMT_LEFT, 160, -1);

	CImageList img2;
	CBitmap arse;

	arse.LoadBitmap(IDB_LIST_RESULTS);
	img2.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 0);
	img2.Add(&arse, RGB(0, 128, 128));
	m_result_list.SetImageList(&img2, LVSIL_SMALL);
	img2.Detach();
	arse.Detach();

	ChangeView();

	if(SearchRConfig::winpos_search_results.Width() > 0)
		// restore the old window pos
		MoveWindow(SearchRConfig::winpos_search_results, false);
	else
	{
		// centre in the parent window
		CRect r_parent, r_child;
		GetParent()->GetWindowRect(r_parent);
		GetWindowRect(r_child);
		MoveWindow(r_parent.right - r_child.Width() - 64, r_parent.top + 128, r_child.Width(), r_child.Height(), false);
	}

	return TRUE;
}

//#ifndef DWORD_PTR
#define DWORD_PTR DWORD
//#endif

void CSearchResults::UpdateSearchResultsList(void)
{
	if(search_id == p_doc->search_id)
		return;

	search_id = p_doc->search_id;

	m_result_list.DeleteAllItems();
	m_result_list.SetItemCount(p_doc->result_count);

	CSearchResult *result_data;

	result_data = p_doc->result_data;

	DWORD i;

	i = 0;

	CString s;

	while(result_data)
	{
		s.Format(_T("%08X"), result_data->offset);
		m_result_list.InsertItem(i, s);
		if(result_data->a_found)
			s.Format(_T("%02X"), result_data->byte_a & 255);
		else
			s = _T("??");
		m_result_list.SetItemText(i, 1, s);
		if(result_data->A_found)
			s.Format(_T("%02X"), result_data->byte_A & 255);
		else
			s = _T("??");
		m_result_list.SetItemText(i, 2, s);
		if(result_data->found_0)
			s.Format(_T("%02X"), result_data->byte_0 & 255);
		else
			s = _T("??");
		m_result_list.SetItemText(i, 3, s);
		{
			DWORD i, j;
			_TCHAR str[32];
			_TCHAR *b = &str[0];
			BYTE *data = p_doc->data + result_data->offset;
			j = result_data->offset + 16;
			if(j > p_doc->size)
				j = p_doc->size;
			for(i = result_data->offset; i < j; i ++, data ++)
				if(result_data->lookup_valid[*data])
					*b++ = result_data->lookup[*data];
				else
					*b++ = '.';
			*b = 0;
			s.Format(_T("%s"), str);
		}
		m_result_list.SetItemText(i, 4, s);
		m_result_list.SetItemData(i, (DWORD_PTR) result_data);
		result_data = result_data->next;
		i ++;
	}

	m_result_list.RedrawWindow();
}

void CSearchResults::OnEditSearch()
{
	CSearchDialog p_dlg;

	if(p_dlg.DoModal() != IDOK)
		return;

	p_doc->DoSearchRelative(p_dlg.m_search_data, p_dlg.m_next_char, 0);
	ChangeView();
}

void CSearchResults::OnBnClickedSrView()
{
	POSITION pos;
	
	pos = m_result_list.GetFirstSelectedItemPosition();
	if(pos)
	{
		DWORD i = m_result_list.GetNextSelectedItem(pos);
		p_view_2->result_data = (CSearchResult *) m_result_list.GetItemData(i);
		p_view_2->ViewResult();
		ChangeView();
	}
}

void CSearchResults::OnNMClickSearchResultList(NMHDR *pNMHDR, LRESULT *pResult)
{
	ChangeView();
	*pResult = 0;
}

void CSearchResults::ChangeView()
{
	UpdateSearchResultsList();
	m_view_result.EnableWindow(m_result_list.GetSelectedCount());
	m_add_to_table.EnableWindow(m_result_list.GetSelectedCount());
	m_new_search.EnableWindow(p_doc->data != NULL);
	//RedrawWindow();
}

void CSearchResults::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	if(bShow)
	{
		ChangeView();
	}
}


void CSearchResults::ClearResults(void)
{
	m_result_list.DeleteAllItems();
}

void CSearchResults::OnBnClickedSrAddToTable()
{
	POSITION pos;
	
	pos = m_result_list.GetFirstSelectedItemPosition();
	DWORD i = m_result_list.GetNextSelectedItem(pos);

	CSearchResult *result;
	
	result = (CSearchResult *) m_result_list.GetItemData(i);

	CSearchAddToTable p_dlg;

	p_dlg.m_enable_lower_case = result->a_found;
	p_dlg.m_enable_upper_case = result->A_found;
	p_dlg.m_enable_numerical = result->found_0;

	if(p_dlg.DoModal() != IDOK)
		return;

	DWORD j;

	for(i = 0; i < 256; i ++)
	{
		if(result->lookup_valid[i] == TRUE)
		{
			j = result->lookup[i];
			if((j >= 'a') && (j <= 'z') && (p_dlg.m_add_lower_case == TRUE))
				p_table_doc->AddToTable((BYTE) i, (BYTE) j);
			if((j >= 'A') && (j <= 'Z') && (p_dlg.m_add_upper_case == TRUE))
				p_table_doc->AddToTable((BYTE) i, (BYTE) j);
			if((j >= '0') && (j <= '9') && (p_dlg.m_add_numerical == TRUE))
				p_table_doc->AddToTable((BYTE) i, (BYTE) j);
		}
	}
}

void CSearchResults::OnDblclkSearchResultList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnBnClickedSrView();
	*pResult = 0;
}

void CSearchResults::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CDialog::OnWindowPosChanged(lpwndpos);
	// store the new window pos
	GetWindowRect(SearchRConfig::winpos_search_results);
}
