// TableDoc.cpp : implementation file
//

#include "stdafx.h"
#include "SearchR.h"
#include "TableDoc.h"
#include "MainFrm.h"
#include "TableCodePage.h"

#define CP_SHIFT_JIS	932
#define CP_EUC_JIS		20932

CTableDoc *p_table_doc;

// CTableDoc

IMPLEMENT_DYNCREATE(CTableDoc, CDocument)

CTableDoc::CTableDoc()
{
	int i;

	for(i = 0; i < MAX_TABLE_SIZE; i ++)
	{
		table_data_mte[i].Empty();
		table_data_reverse[i] = 0;
		table_valid[i] = FALSE;
	}

	filename = _T("table1.tbl");

	SetModifiedFlag(FALSE);
}

BOOL CTableDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CTableDoc::~CTableDoc()
{
}


BEGIN_MESSAGE_MAP(CTableDoc, CDocument)
END_MESSAGE_MAP()


// CTableDoc diagnostics

#ifdef _DEBUG
void CTableDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CTableDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CTableDoc serialization

void CTableDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}


// CTableDoc commands

BOOL CTableDoc::SaveTable(void)
{
	if(!IsModified())
		return TRUE;

	CString question;

	question.Format(_T("Save changes to %s?"), filename);

	DWORD reply = AfxMessageBox(question, MB_YESNOCANCEL, 0);

	if(reply == IDNO)
		return TRUE;

	if(reply == IDCANCEL)
		return FALSE;

	OPENFILENAME ofn;
	_TCHAR fn[1024];

	_tcscpy(fn, (LPCTSTR) filename);

	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = p_mainframe->m_hWnd;
	ofn.lpstrFile = fn;
	ofn.nMaxFile = sizeof(fn);
	ofn.lpstrFilter = _T("Table Files (*.tbl)\0*.tbl\0All Files (*.*)\0*.*\0");
	ofn.nFilterIndex = 0;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST;

	if(!GetSaveFileName(&ofn))
		return FALSE;

	DWORD i;
	CStdioFile f;
	CString s;

	if(!f.Open( ofn.lpstrFile, CFile::modeCreate | CFile::modeWrite | CFile::typeText ) )
		return TRUE;

	for(i = 0; i < 256; i ++)
	{
		if(table_valid[i])
		{
			//if(_istprint(table_data[i]))
			{
				s.Format(_T("%02X=%s\n"), i, table_data_mte[i]);
				f.WriteString(s);
			}
		}
	}

	CBookmark* curr_bookmark;

	CString str_bookmark;

	POSITION pos;

	pos = p_table_doc->m_bookmark_list.GetHeadPosition();

	while(pos != NULL)
	{
		curr_bookmark = (CBookmark *) p_table_doc->m_bookmark_list.GetAt(pos);

		str_bookmark.Format(_T("(%08Xh)%s\n"), curr_bookmark->m_offset, curr_bookmark->m_name);
		f.WriteString(str_bookmark);

		p_table_doc->m_bookmark_list.GetNext(pos);
	}

	f.Close();
	return TRUE;
}

void CTableDoc::AddToTable(DWORD offset, _TCHAR data)
{
	offset &= 0xFFFF;
	table_data_mte[offset] = (char) data;
	table_data_reverse[data] = offset;
	table_valid[offset] = TRUE;

	SetModifiedFlag(TRUE);
}

void CTableDoc::AddToTable(DWORD offset, LPCTSTR data)
{
	offset &= 0xFFFF;
	table_data_mte[offset] = data;
	table_valid[offset] = TRUE;

	SetModifiedFlag(TRUE);
}

void CTableDoc::DeleteFromTable(DWORD offset)
{
	offset &= 0xFFFF;
	table_valid[offset] = FALSE;

	SetModifiedFlag(TRUE);
}

void CTableDoc::LoadTable(void)
{
	OPENFILENAME ofn;
	_TCHAR file_path[1024];
	_TCHAR file_title[1024];

	_tcscpy(file_path, _T(""));
	_tcscpy(file_title, _T(""));

	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = p_mainframe->m_hWnd;
	ofn.lpstrFile = file_path;
	ofn.nMaxFile = sizeof(file_path);
	ofn.lpstrFilter = _T("Table Files (*.tbl)\0*.tbl\0All Files (*.*)\0*.*\0");
	ofn.nFilterIndex = 0;
	ofn.lpstrFileTitle = file_title;
	ofn.nMaxFileTitle = sizeof(file_title);
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if(!GetOpenFileName(&ofn))
		return;

	DoLoadTable(ofn.lpstrFile, ofn.lpstrFileTitle);
}

void CTableDoc::DoLoadTable(LPCTSTR file_name, LPCTSTR file_title)
{
	DWORD i;
	CStdioFile f;
	CString s;
	CString data;
	UINT code_page;

	if(!f.Open( file_name, CFile::modeRead | CFile::typeText ) )
		return;

	filename = file_title;

	{
		CTableCodePage p_table_codepage;

		if(p_table_codepage.DoModal() == IDOK)
		{
			if(p_table_codepage.m_code_page == 1)
				code_page = CP_SHIFT_JIS;
			else
				if(p_table_codepage.m_code_page == 2)
					code_page = CP_EUC_JIS;
				else
					code_page = CP_ACP;
		}
		else
			return;
	}

	{
		CString s[SearchRConfig::TABLE_MRU_SIZE];
		DWORD i, j;

		for(i = 0; i < SearchRConfig::TABLE_MRU_SIZE; i ++)
			s[i] = SearchRConfig::table_mru_list[i];

		SearchRConfig::table_mru_list[0] = file_name;

		for(i = 0, j = 1; (i < SearchRConfig::TABLE_MRU_SIZE) && (j < SearchRConfig::TABLE_MRU_SIZE); i ++)
		{
			if(s[i] != file_name)
				if(s[i].IsEmpty() == FALSE)
					SearchRConfig::table_mru_list[j++] = s[i];
		}
		while(j < SearchRConfig::TABLE_MRU_SIZE)
			SearchRConfig::table_mru_list[j++].Empty();
	}
	p_mainframe->UpdateTableList();

	while(f.ReadString(s))
	{
		if(!s.IsEmpty())
		{
			if(s[0] == '/')
			{
				// assume it's an end of line marker

				_stscanf(s.Right(s.GetLength() - 1), _T("%X"), &i);

				AddToTable(i, _T(""));
			}
			else
			{
				if(s[0] == '*')
				{
					// assume it's an end of string marker

					_stscanf(s.Right(s.GetLength() - 1), _T("%X"), &i);

					AddToTable(i, _T(""));
				}
				else
				{
					if(s[0] == '(')
					{
						// assume it's a bookmark

						if(_tcschr(s, ')'))
						{
							s.TrimLeft(_T("("));
							if(_tcsstr(s, _T("h)")))
							{
								_stscanf(s, _T("%X"), &i);
							}
							else
							{
								_stscanf(s, _T("%d"), &i);
							}
						}

						AddBookmark(i, _tcschr(s, ')') + 1);
					}
					else
					{
						if(s[2] == '=')
						{
							// assume it's a 1 byte table entry

							_stscanf(s, _T("%02X"), &i);

							data = s.Right(s.GetLength() - 3);

							{
								// convert the unicode buffer back into bytes
								CHAR *data3 = new CHAR[data.GetLength() + 2];
								int ii;
								for(ii = 0; ii < data.GetLength(); ii ++)
								{
									data3[ii] = data.GetAt(ii);
								}
								data3[data.GetLength() + 0] = 0;
								data3[data.GetLength() + 1] = 0;

								// we want to convert an MBCS string in lpszA
								int nLen = MultiByteToWideChar(code_page, 0, data3, -1, NULL, NULL);
								if(nLen != 0)
								{
									LPWSTR data2 = new WCHAR[nLen];
									MultiByteToWideChar(code_page, 0, data3, -1, data2, nLen);
									// use it to call OLE here
									AddToTable(i, data2);
									// free the string
									delete[] data2;
								}
								delete[] data3;
							}
						}
						else
						{
							if(s[4] == '=')
							{
								// assume it's a 2 byte table entry

								_stscanf(s, _T("%04X"), &i);

								data = s.Right(s.GetLength() - 5);

								{
									// convert the unicode buffer back into bytes
									CHAR *data3 = new CHAR[data.GetLength() + 2];
									int ii;
									for(ii = 0; ii < data.GetLength(); ii ++)
									{
										data3[ii] = data.GetAt(ii);
									}
									data3[data.GetLength() + 0] = 0;
									data3[data.GetLength() + 1] = 0;

									// we want to convert an MBCS string in lpszA
									int nLen = MultiByteToWideChar(code_page, 0, data3, -1, NULL, NULL);
									if(nLen != 0)
									{
										LPWSTR data2 = new WCHAR[nLen];
										MultiByteToWideChar(code_page, 0, data3, -1, data2, nLen);
										// use it to call OLE here
										AddToTable(i, data2);
										// free the string
										delete[] data2;
									}
									delete[] data3;
								}
							}
						}
					}
				}
			}
		}
	}

	SetModifiedFlag(FALSE);

	f.Close();

	p_mainframe->UpdateBookmarks();
	p_mainframe->UpdateCurrentView();

}

void CTableDoc::AddBookmark(DWORD offset, LPCTSTR name)
{
	CBookmark* curr_bookmark;

	POSITION pos;

	pos = m_bookmark_list.GetHeadPosition();

	while(pos != NULL)
	{
		curr_bookmark = (CBookmark *) m_bookmark_list.GetAt(pos);

		if(curr_bookmark->m_offset == offset)
		{
			// update an existing bookmark
			curr_bookmark->m_name = name;
			SetModifiedFlag(TRUE);
			return;
		}

		if(curr_bookmark->m_offset > offset)
		{
			// insert in ascending order
			CBookmark *new_bookmark = new CBookmark;

			new_bookmark->m_offset = offset;
			new_bookmark->m_name = name;

			m_bookmark_list.InsertBefore(pos, new_bookmark);
			SetModifiedFlag(TRUE);
			return;
		}

		m_bookmark_list.GetNext(pos);
	}

	// insert at the end of the list
	CBookmark *new_bookmark = new CBookmark;

	new_bookmark->m_offset = offset;
	new_bookmark->m_name = name;

	m_bookmark_list.AddTail(new_bookmark);
	SetModifiedFlag(TRUE);
}

void CTableDoc::DeleteBookmark(DWORD offset)
{
	CBookmark* curr_bookmark;

	POSITION pos;

	pos = m_bookmark_list.GetHeadPosition();

	while(pos != NULL)
	{
		curr_bookmark = (CBookmark *) m_bookmark_list.GetAt(pos);

		if(curr_bookmark->m_offset == offset)
		{
			// delete an existing bookmark
			m_bookmark_list.RemoveAt(pos);
			SetModifiedFlag(TRUE);
			delete curr_bookmark;
			return;
		}

		m_bookmark_list.GetNext(pos);
	}
}
