// TableInfo.cpp : implementation file
//

#include "stdafx.h"
#include "searchr.h"
#include "TableDoc.h"
#include "SearchRDoc.h"
#include "TableInfo.h"
#include "HexEditView.h"
#include "TableView.h"
#include "TableAdd.h"

// CTableInfo dialog

IMPLEMENT_DYNAMIC(CTableInfo, CDialog)

CTableInfo::CTableInfo(CWnd* pParent /*=NULL*/)
	: CDialog(CTableInfo::IDD, pParent)
{
	ASSERT(pParent != NULL);

	m_pParent = pParent;
	m_nID = CTableInfo::IDD;
}

CTableInfo::~CTableInfo()
{
}


void CTableInfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_TABLE_LIST, m_table_list);
	DDX_Control(pDX, IDC_TABLE_ADD, m_table_add);
	DDX_Control(pDX, IDC_TABLE_EDIT, m_table_edit);
	DDX_Control(pDX, ID_TABLE_DELETE, m_table_delete);
}


BEGIN_MESSAGE_MAP(CTableInfo, CDialog)
	//{{AFX_MSG_MAP(CTableInfo)
	ON_BN_CLICKED(IDC_TABLE_ADD, OnBnClickedTableAdd)
	ON_BN_CLICKED(IDC_TABLE_EDIT, OnBnClickedTableEdit)
	ON_BN_CLICKED(ID_TABLE_DELETE, OnBnClickedTableDelete)
	ON_NOTIFY(NM_CLICK, IDC_TABLE_LIST, OnNMClickTableList)
	//}}AFX_MSG_MAP
	ON_WM_WINDOWPOSCHANGED()
END_MESSAGE_MAP()

// CTableInfo message handlers

BOOL CTableInfo::Create()
{
	return CDialog::Create(m_nID, m_pParent);
}

BOOL CTableInfo::OnInitDialog()
{
	CDialog::OnInitDialog();
/*
	LOGFONT lf;
	
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight = 16; //10;
	//_tcscpy(lf.lfFaceName, _T("Courier"));
	_tcscpy(lf.lfFaceName, _T("MS Gothic"));
	m_font.CreateFontIndirect(&lf);
	m_table_list.SetFont(&m_font);
*/

	m_font.CreateFont ( 16, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_MODERN, _T("MS Gothic") );
	m_table_list.SetFont(&m_font);
	
	m_table_list.InsertColumn(0, _T("Hex"), LVCFMT_LEFT, 64, -1);
	m_table_list.InsertColumn(1, _T("ASCII"), LVCFMT_LEFT, 64, -1);

	CImageList img2;
	CBitmap arse;

	arse.LoadBitmap(IDB_LIST_TABLE);
	img2.Create(16, 16, ILC_COLOR32 | ILC_MASK, 0, 0);
	img2.Add(&arse, RGB(0, 128, 128));
	m_table_list.SetImageList(&img2, LVSIL_SMALL);
	img2.Detach();
	arse.Detach();

	UpdateList();
	ChangeView();

	if(SearchRConfig::winpos_table_info.Width() > 0)
		// restore the old window pos
		MoveWindow(SearchRConfig::winpos_table_info, false);
	else
	{
		// centre in the parent window
		CRect r_parent, r_child;
		GetParent()->GetWindowRect(r_parent);
		GetWindowRect(r_child);
		MoveWindow(r_parent.right - r_child.Width() - 64, r_parent.top + 128, r_child.Width(), r_child.Height(), false);
	}

	return TRUE;
}

void CTableInfo::ChangeView(void)
{
	m_table_edit.EnableWindow(m_table_list.GetSelectedCount());
	m_table_delete.EnableWindow(m_table_list.GetSelectedCount());
}

void CTableInfo::UpdateList(void)
{
	m_table_list.DeleteAllItems();

	DWORD i, j;
	CString s;

	if(p_table_doc)
	{
		j = 0;
		for(i = 0; i < CTableDoc::MAX_TABLE_SIZE; i ++)
		{
			if(p_table_doc->table_valid[i])
			{
				if(i < 0x100)
					s.Format(_T("%02X"), i);
				else
					s.Format(_T("%04X"), i);
				m_table_list.InsertItem(j, s);
				//s = (char) (_istprint(p_table_doc->table_data[i]) ? p_table_doc->table_data[i] : '.');
				s = p_table_doc->table_data_mte[i];
				m_table_list.SetItemText(j, 1, s);
				m_table_list.SetItemData(j, i);
				j ++;
			}
		}
	}
}

void CTableInfo::OnBnClickedTableAdd()
{
	CTableAdd p_table_add;

	if(p_table_add.DoModal() == IDOK)
	{
		if((p_table_add.m_add_offset.IsEmpty() == FALSE) && (p_table_add.m_add_data.IsEmpty() == FALSE))
		{
			if(p_table_doc)
			{
				DWORD i = m_table_list.GetItemCount();
				DWORD j;
				_stscanf(p_table_add.m_add_offset, _T("%X"), &j);
				CString s;
				if(j < 0x100)
					s.Format(_T("%02X"), j);
				else
					s.Format(_T("%04X"), j);
				m_table_list.InsertItem(i, s);
				m_table_list.SetItemText(i, 1, p_table_add.m_add_data);
				m_table_list.SetItemData(i, j);

				p_table_doc->AddToTable(j, p_table_add.m_add_data);
			}
		}
	}

	ChangeView();
}

void CTableInfo::OnBnClickedTableEdit()
{
	POSITION pos;
	
	pos = m_table_list.GetFirstSelectedItemPosition();
	DWORD i = m_table_list.GetNextSelectedItem(pos);

	CTableAdd p_table_add;

	if(m_table_list.GetItemData(i) < 0x100)
		p_table_add.m_add_offset.Format(_T("%02X"), m_table_list.GetItemData(i));
	else
		p_table_add.m_add_offset.Format(_T("%04X"), m_table_list.GetItemData(i));
	p_table_add.m_add_data = m_table_list.GetItemText(i, 1);

	if(p_table_add.DoModal() == IDOK)
	{
		if((p_table_add.m_add_offset.IsEmpty() == FALSE) && (p_table_add.m_add_data.IsEmpty() == FALSE))
		{
			if(p_table_doc)
			{
				DWORD j;
				_stscanf(p_table_add.m_add_offset, _T("%X"), &j);
				CString s;
				if(j < 0x100)
					s.Format(_T("%02X"), j);
				else
					s.Format(_T("%04X"), j);
				m_table_list.SetItemText(i, 0, s);
				m_table_list.SetItemText(i, 1, p_table_add.m_add_data);
				m_table_list.SetItemData(i, j);

				p_table_doc->DeleteFromTable(j);
				p_table_doc->AddToTable(j, p_table_add.m_add_data);
			}
		}
	}

	ChangeView();
}

void CTableInfo::OnBnClickedTableDelete()
{
	POSITION pos;
	
	while(pos = m_table_list.GetFirstSelectedItemPosition())
	{
		int i = m_table_list.GetNextSelectedItem(pos);
		p_table_doc->DeleteFromTable(m_table_list.GetItemData(i));
		m_table_list.DeleteItem(i);
	}

	ChangeView();
}

void CTableInfo::OnNMClickTableList(NMHDR *pNMHDR, LRESULT *pResult)
{
	ChangeView();
	*pResult = 0;
}

void CTableInfo::OnWindowPosChanged(WINDOWPOS* lpwndpos)
{
	CDialog::OnWindowPosChanged(lpwndpos);
	// store the new window pos
	GetWindowRect(SearchRConfig::winpos_table_info);
}
